# BEGIN GENERATED CONTENT (do not edit below this line)

# This content is generated by gengl.py.
# Wrapper for /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h
from OpenGL import platform, constant
from ctypes import *
c_void = None


CONST = constant.Constant( 'CONST', 0 )
GLenum = c_uint 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:19
GLboolean = c_ubyte 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:20
GLbitfield = c_uint 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:21
GLbyte = c_char 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:22
GLshort = c_short 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:23
GLint = c_int 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:24
GLsizei = c_int 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:25
GLubyte = c_ubyte 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:26
GLushort = c_ushort 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:27
GLuint = c_uint 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:28
GLfloat = c_float 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:29
GLclampf = c_float 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:30
GLdouble = c_double 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:31
GLclampd = c_double 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:32
GLvoid = None 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:33
INT8 = c_char 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:35
PINT8 = c_char_p 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:35
INT16 = c_short 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:36
PINT16 = POINTER(c_short) 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:36
INT32 = c_int 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:37
PINT32 = POINTER(c_int) 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:37
UINT8 = c_ubyte 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:38
PUINT8 = POINTER(c_ubyte) 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:38
UINT16 = c_ushort 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:39
PUINT16 = POINTER(c_ushort) 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:39
UINT32 = c_uint 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:40
PUINT32 = POINTER(c_uint) 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:40
LONG32 = c_int 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:41
PLONG32 = POINTER(c_int) 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:41
ULONG32 = c_uint 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:42
PULONG32 = POINTER(c_uint) 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:42
DWORD32 = c_uint 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:43
PDWORD32 = POINTER(c_uint) 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:43
INT64 = c_longlong 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:44
PINT64 = POINTER(c_longlong) 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:44
UINT64 = c_ulonglong 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:45
PUINT64 = POINTER(c_ulonglong) 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:45
VOID = None 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:47
LPVOID = POINTER(None) 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:47
LPCSTR = c_char_p 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:48
CHAR = c_char 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:49
BYTE = c_ubyte 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:50
WORD = c_ushort 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:51
USHORT = c_ushort 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:51
UINT = c_uint 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:52
INT = c_int 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:53
INT_PTR = POINTER(c_int) 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:53
BOOL = c_long 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:54
LONG = c_long 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:55
DWORD = c_ulong 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:56
FLOAT = c_float 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:57
COLORREF = DWORD 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:58
LPCOLORREF = POINTER(DWORD) 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:58

HANDLE = POINTER(None) 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:60
# TODO: figure out how to make the handle not appear as a void_p within the code...
HANDLE.final = True

HGLRC = HANDLE 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:62
HDC = HANDLE 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:63
PROC = CFUNCTYPE(INT_PTR) 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:65
wglCopyContext = platform.createBaseFunction(
    'wglCopyContext', dll=platform.GL, resultType=BOOL, 
    argTypes=[HGLRC, HGLRC, UINT],
    doc='wglCopyContext( HGLRC(None), HGLRC(None), UINT(None) ) -> BOOL', 
    argNames=['None', 'None', 'None'],
)

wglCreateContext = platform.createBaseFunction(
    'wglCreateContext', dll=platform.GL, resultType=HGLRC, 
    argTypes=[HDC],
    doc='wglCreateContext( HDC(None) ) -> HGLRC', 
    argNames=['None'],
)

wglCreateLayerContext = platform.createBaseFunction(
    'wglCreateLayerContext', dll=platform.GL, resultType=HGLRC, 
    argTypes=[HDC, c_int],
    doc='wglCreateLayerContext( HDC(None), c_int(None) ) -> HGLRC', 
    argNames=['None', 'None'],
)

wglDeleteContext = platform.createBaseFunction(
    'wglDeleteContext', dll=platform.GL, resultType=BOOL, 
    argTypes=[HGLRC],
    doc='wglDeleteContext( HGLRC(None) ) -> BOOL', 
    argNames=['None'],
)

wglGetCurrentContext = platform.createBaseFunction(
    'wglGetCurrentContext', dll=platform.GL, resultType=HGLRC, 
    argTypes=[],
    doc='wglGetCurrentContext(  ) -> HGLRC', 
    argNames=[],
)

wglGetCurrentDC = platform.createBaseFunction(
    'wglGetCurrentDC', dll=platform.GL, resultType=HDC, 
    argTypes=[],
    doc='wglGetCurrentDC(  ) -> HDC', 
    argNames=[],
)

wglGetProcAddress = platform.createBaseFunction(
    'wglGetProcAddress', dll=platform.GL, resultType=PROC, 
    argTypes=[LPCSTR],
    doc='wglGetProcAddress( LPCSTR(None) ) -> PROC', 
    argNames=['None'],
)

wglMakeCurrent = platform.createBaseFunction(
    'wglMakeCurrent', dll=platform.GL, resultType=BOOL, 
    argTypes=[HDC, HGLRC],
    doc='wglMakeCurrent( HDC(None), HGLRC(None) ) -> BOOL', 
    argNames=['None', 'None'],
)

wglShareLists = platform.createBaseFunction(
    'wglShareLists', dll=platform.GL, resultType=BOOL, 
    argTypes=[HGLRC, HGLRC],
    doc='wglShareLists( HGLRC(None), HGLRC(None) ) -> BOOL', 
    argNames=['None', 'None'],
)
wglUseFontBitmapsA = platform.createBaseFunction(
    'wglUseFontBitmapsA', dll=platform.GL, resultType=BOOL, 
    argTypes=[HDC, DWORD, DWORD, DWORD],
    doc='wglUseFontBitmapsA( HDC(None), DWORD(None), DWORD(None), DWORD(None) ) -> BOOL', 
    argNames=['None', 'None', 'None', 'None'],
)

wglUseFontBitmapsW = platform.createBaseFunction(
    'wglUseFontBitmapsW', dll=platform.GL, resultType=BOOL, 
    argTypes=[HDC, DWORD, DWORD, DWORD],
    doc='wglUseFontBitmapsW( HDC(None), DWORD(None), DWORD(None), DWORD(None) ) -> BOOL', 
    argNames=['None', 'None', 'None', 'None'],
)

SwapBuffers = platform.createBaseFunction(
    'SwapBuffers', dll=platform.GL, resultType=BOOL, 
    argTypes=[HDC],
    doc='SwapBuffers( HDC(None) ) -> BOOL', 
    argNames=['None'],
)

class struct__POINTFLOAT(Structure):
    __slots__ = [
        'x',
        'y',
    ]
struct__POINTFLOAT._fields_ = [
    ('x', FLOAT),
    ('y', FLOAT),
]

POINTFLOAT = struct__POINTFLOAT 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:83
PPOINTFLOAT = POINTER(struct__POINTFLOAT) 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:83
class struct__GLYPHMETRICSFLOAT(Structure):
    __slots__ = [
        'gmfBlackBoxX',
        'gmfBlackBoxY',
        'gmfptGlyphOrigin',
        'gmfCellIncX',
        'gmfCellIncY',
    ]
struct__GLYPHMETRICSFLOAT._fields_ = [
    ('gmfBlackBoxX', FLOAT),
    ('gmfBlackBoxY', FLOAT),
    ('gmfptGlyphOrigin', POINTFLOAT),
    ('gmfCellIncX', FLOAT),
    ('gmfCellIncY', FLOAT),
]

GLYPHMETRICSFLOAT = struct__GLYPHMETRICSFLOAT 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:91
PGLYPHMETRICSFLOAT = POINTER(struct__GLYPHMETRICSFLOAT) 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:91
LPGLYPHMETRICSFLOAT = POINTER(struct__GLYPHMETRICSFLOAT) 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:91
WGL_FONT_LINES = constant.Constant( 'WGL_FONT_LINES', 0 )
WGL_FONT_POLYGONS = constant.Constant( 'WGL_FONT_POLYGONS', 1 )
wglUseFontOutlinesA = platform.createBaseFunction(
    'wglUseFontOutlinesA', dll=platform.GL, resultType=BOOL, 
    argTypes=[HDC, DWORD, DWORD, DWORD, FLOAT, FLOAT, c_int, LPGLYPHMETRICSFLOAT],
    doc='wglUseFontOutlinesA( HDC(None), DWORD(None), DWORD(None), DWORD(None), FLOAT(None), FLOAT(None), c_int(None), LPGLYPHMETRICSFLOAT(None) ) -> BOOL', 
    argNames=['None', 'None', 'None', 'None', 'None', 'None', 'None', 'None'],
)

wglUseFontOutlinesW = platform.createBaseFunction(
    'wglUseFontOutlinesW', dll=platform.GL, resultType=BOOL, 
    argTypes=[HDC, DWORD, DWORD, DWORD, FLOAT, FLOAT, c_int, LPGLYPHMETRICSFLOAT],
    doc='wglUseFontOutlinesW( HDC(None), DWORD(None), DWORD(None), DWORD(None), FLOAT(None), FLOAT(None), c_int(None), LPGLYPHMETRICSFLOAT(None) ) -> BOOL', 
    argNames=['None', 'None', 'None', 'None', 'None', 'None', 'None', 'None'],
)

class struct_tagLAYERPLANEDESCRIPTOR(Structure):
    __slots__ = [
        'nSize',
        'nVersion',
        'dwFlags',
        'iPixelType',
        'cColorBits',
        'cRedBits',
        'cRedShift',
        'cGreenBits',
        'cGreenShift',
        'cBlueBits',
        'cBlueShift',
        'cAlphaBits',
        'cAlphaShift',
        'cAccumBits',
        'cAccumRedBits',
        'cAccumGreenBits',
        'cAccumBlueBits',
        'cAccumAlphaBits',
        'cDepthBits',
        'cStencilBits',
        'cAuxBuffers',
        'iLayerPlane',
        'bReserved',
        'crTransparent',
    ]
struct_tagLAYERPLANEDESCRIPTOR._fields_ = [
    ('nSize', WORD),
    ('nVersion', WORD),
    ('dwFlags', DWORD),
    ('iPixelType', BYTE),
    ('cColorBits', BYTE),
    ('cRedBits', BYTE),
    ('cRedShift', BYTE),
    ('cGreenBits', BYTE),
    ('cGreenShift', BYTE),
    ('cBlueBits', BYTE),
    ('cBlueShift', BYTE),
    ('cAlphaBits', BYTE),
    ('cAlphaShift', BYTE),
    ('cAccumBits', BYTE),
    ('cAccumRedBits', BYTE),
    ('cAccumGreenBits', BYTE),
    ('cAccumBlueBits', BYTE),
    ('cAccumAlphaBits', BYTE),
    ('cDepthBits', BYTE),
    ('cStencilBits', BYTE),
    ('cAuxBuffers', BYTE),
    ('iLayerPlane', BYTE),
    ('bReserved', BYTE),
    ('crTransparent', COLORREF),
]

LAYERPLANEDESCRIPTOR = struct_tagLAYERPLANEDESCRIPTOR 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:127
PLAYERPLANEDESCRIPTOR = POINTER(struct_tagLAYERPLANEDESCRIPTOR) 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:127
LPLAYERPLANEDESCRIPTOR = POINTER(struct_tagLAYERPLANEDESCRIPTOR) 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:127
LPD_DOUBLEBUFFER = constant.Constant( 'LPD_DOUBLEBUFFER', 1 )
LPD_STEREO = constant.Constant( 'LPD_STEREO', 2 )
LPD_SUPPORT_GDI = constant.Constant( 'LPD_SUPPORT_GDI', 16 )
LPD_SUPPORT_OPENGL = constant.Constant( 'LPD_SUPPORT_OPENGL', 32 )
LPD_SHARE_DEPTH = constant.Constant( 'LPD_SHARE_DEPTH', 64 )
LPD_SHARE_STENCIL = constant.Constant( 'LPD_SHARE_STENCIL', 128 )
LPD_SHARE_ACCUM = constant.Constant( 'LPD_SHARE_ACCUM', 256 )
LPD_SWAP_EXCHANGE = constant.Constant( 'LPD_SWAP_EXCHANGE', 512 )
LPD_SWAP_COPY = constant.Constant( 'LPD_SWAP_COPY', 1024 )
LPD_TRANSPARENT = constant.Constant( 'LPD_TRANSPARENT', 4096 )
LPD_TYPE_RGBA = constant.Constant( 'LPD_TYPE_RGBA', 0 )
LPD_TYPE_COLORINDEX = constant.Constant( 'LPD_TYPE_COLORINDEX', 1 )
WGL_SWAP_MAIN_PLANE = constant.Constant( 'WGL_SWAP_MAIN_PLANE', 1 )
WGL_SWAP_OVERLAY1 = constant.Constant( 'WGL_SWAP_OVERLAY1', 2 )
WGL_SWAP_OVERLAY2 = constant.Constant( 'WGL_SWAP_OVERLAY2', 4 )
WGL_SWAP_OVERLAY3 = constant.Constant( 'WGL_SWAP_OVERLAY3', 8 )
WGL_SWAP_OVERLAY4 = constant.Constant( 'WGL_SWAP_OVERLAY4', 16 )
WGL_SWAP_OVERLAY5 = constant.Constant( 'WGL_SWAP_OVERLAY5', 32 )
WGL_SWAP_OVERLAY6 = constant.Constant( 'WGL_SWAP_OVERLAY6', 64 )
WGL_SWAP_OVERLAY7 = constant.Constant( 'WGL_SWAP_OVERLAY7', 128 )
WGL_SWAP_OVERLAY8 = constant.Constant( 'WGL_SWAP_OVERLAY8', 256 )
WGL_SWAP_OVERLAY9 = constant.Constant( 'WGL_SWAP_OVERLAY9', 512 )
WGL_SWAP_OVERLAY10 = constant.Constant( 'WGL_SWAP_OVERLAY10', 1024 )
WGL_SWAP_OVERLAY11 = constant.Constant( 'WGL_SWAP_OVERLAY11', 2048 )
WGL_SWAP_OVERLAY12 = constant.Constant( 'WGL_SWAP_OVERLAY12', 4096 )
WGL_SWAP_OVERLAY13 = constant.Constant( 'WGL_SWAP_OVERLAY13', 8192 )
WGL_SWAP_OVERLAY14 = constant.Constant( 'WGL_SWAP_OVERLAY14', 16384 )
WGL_SWAP_OVERLAY15 = constant.Constant( 'WGL_SWAP_OVERLAY15', 32768 )
WGL_SWAP_UNDERLAY1 = constant.Constant( 'WGL_SWAP_UNDERLAY1', 65536 )
WGL_SWAP_UNDERLAY2 = constant.Constant( 'WGL_SWAP_UNDERLAY2', 131072 )
WGL_SWAP_UNDERLAY3 = constant.Constant( 'WGL_SWAP_UNDERLAY3', 262144 )
WGL_SWAP_UNDERLAY4 = constant.Constant( 'WGL_SWAP_UNDERLAY4', 524288 )
WGL_SWAP_UNDERLAY5 = constant.Constant( 'WGL_SWAP_UNDERLAY5', 1048576 )
WGL_SWAP_UNDERLAY6 = constant.Constant( 'WGL_SWAP_UNDERLAY6', 2097152 )
WGL_SWAP_UNDERLAY7 = constant.Constant( 'WGL_SWAP_UNDERLAY7', 4194304 )
WGL_SWAP_UNDERLAY8 = constant.Constant( 'WGL_SWAP_UNDERLAY8', 8388608 )
WGL_SWAP_UNDERLAY9 = constant.Constant( 'WGL_SWAP_UNDERLAY9', 16777216 )
WGL_SWAP_UNDERLAY10 = constant.Constant( 'WGL_SWAP_UNDERLAY10', 33554432 )
WGL_SWAP_UNDERLAY11 = constant.Constant( 'WGL_SWAP_UNDERLAY11', 67108864 )
WGL_SWAP_UNDERLAY12 = constant.Constant( 'WGL_SWAP_UNDERLAY12', 134217728 )
WGL_SWAP_UNDERLAY13 = constant.Constant( 'WGL_SWAP_UNDERLAY13', 268435456 )
WGL_SWAP_UNDERLAY14 = constant.Constant( 'WGL_SWAP_UNDERLAY14', 536870912 )
WGL_SWAP_UNDERLAY15 = constant.Constant( 'WGL_SWAP_UNDERLAY15', 1073741824 )
wglDescribeLayerPlane = platform.createBaseFunction(
    'wglDescribeLayerPlane', dll=platform.GL, resultType=BOOL, 
    argTypes=[HDC, c_int, c_int, UINT, LPLAYERPLANEDESCRIPTOR],
    doc='wglDescribeLayerPlane( HDC(None), c_int(None), c_int(None), UINT(None), LPLAYERPLANEDESCRIPTOR(None) ) -> BOOL', 
    argNames=['None', 'None', 'None', 'None', 'None'],
)

wglSetLayerPaletteEntries = platform.createBaseFunction(
    'wglSetLayerPaletteEntries', dll=platform.GL, resultType=c_int, 
    argTypes=[HDC, c_int, c_int, c_int, POINTER(COLORREF)],
    doc='wglSetLayerPaletteEntries( HDC(None), c_int(None), c_int(None), c_int(None), POINTER(COLORREF)() ) -> c_int', 
    argNames=['None', 'None', 'None', 'None', ''],
)

wglGetLayerPaletteEntries = platform.createBaseFunction(
    'wglGetLayerPaletteEntries', dll=platform.GL, resultType=c_int, 
    argTypes=[HDC, c_int, c_int, c_int, POINTER(COLORREF)],
    doc='wglGetLayerPaletteEntries( HDC(None), c_int(None), c_int(None), c_int(None), POINTER(COLORREF)() ) -> c_int', 
    argNames=['None', 'None', 'None', 'None', ''],
)

wglRealizeLayerPalette = platform.createBaseFunction(
    'wglRealizeLayerPalette', dll=platform.GL, resultType=BOOL, 
    argTypes=[HDC, c_int, BOOL],
    doc='wglRealizeLayerPalette( HDC(None), c_int(None), BOOL(None) ) -> BOOL', 
    argNames=['None', 'None', 'None'],
)

wglSwapLayerBuffers = platform.createBaseFunction(
    'wglSwapLayerBuffers', dll=platform.GL, resultType=BOOL, 
    argTypes=[HDC, UINT],
    doc='wglSwapLayerBuffers( HDC(None), UINT(None) ) -> BOOL', 
    argNames=['None', 'None'],
)

class struct__WGLSWAP(Structure):
    __slots__ = [
        'hdc',
        'uiFlags',
    ]
struct__WGLSWAP._fields_ = [
    ('hdc', HDC),
    ('uiFlags', UINT),
]

WGLSWAP = struct__WGLSWAP 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:190
PWGLSWAP = POINTER(struct__WGLSWAP) 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:190
LPWGLSWAP = POINTER(struct__WGLSWAP) 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:190
WGL_SWAPMULTIPLE_MAX = constant.Constant( 'WGL_SWAPMULTIPLE_MAX', 16 )
wglSwapMultipleBuffers = platform.createBaseFunction(
    'wglSwapMultipleBuffers', dll=platform.GL, resultType=DWORD, 
    argTypes=[UINT, POINTER(WGLSWAP)],
    doc='wglSwapMultipleBuffers( UINT(None), POINTER(WGLSWAP)() ) -> DWORD', 
    argNames=['None', ''],
)

class struct_tagRECT(Structure):
    __slots__ = [
        'left',
        'top',
        'right',
        'bottom',
    ]
struct_tagRECT._fields_ = [
    ('left', LONG),
    ('top', LONG),
    ('right', LONG),
    ('bottom', LONG),
]

RECT = struct_tagRECT 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:202
PRECT = POINTER(struct_tagRECT) 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:202
NPRECT = POINTER(struct_tagRECT) 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:202
LPRECT = POINTER(struct_tagRECT) 	# /home/mcfletch/pylive/OpenGL-ctypes/src/wgl.h:202

__all__ = ['CONST', 'GLenum', 'GLboolean', 'GLbitfield', 'GLbyte', 'GLshort',
'GLint', 'GLsizei', 'GLubyte', 'GLushort', 'GLuint', 'GLfloat', 'GLclampf',
'GLdouble', 'GLclampd', 'GLvoid', 'INT8', 'PINT8', 'INT16', 'PINT16', 'INT32',
'PINT32', 'UINT8', 'PUINT8', 'UINT16', 'PUINT16', 'UINT32', 'PUINT32',
'LONG32', 'PLONG32', 'ULONG32', 'PULONG32', 'DWORD32', 'PDWORD32', 'INT64',
'PINT64', 'UINT64', 'PUINT64', 'VOID', 'LPVOID', 'LPCSTR', 'CHAR', 'BYTE',
'WORD', 'USHORT', 'UINT', 'INT', 'INT_PTR', 'BOOL', 'LONG', 'DWORD', 'FLOAT',
'COLORREF', 'LPCOLORREF', 'HANDLE', 'HGLRC', 'HDC', 'PROC', 'wglCopyContext',
'wglCreateContext', 'wglCreateLayerContext', 'wglDeleteContext',
'wglGetCurrentContext', 'wglGetCurrentDC', 'wglGetProcAddress',
'wglMakeCurrent', 'wglShareLists', 'wglUseFontBitmapsA', 'wglUseFontBitmapsW',
'SwapBuffers', 'POINTFLOAT', 'PPOINTFLOAT', 'GLYPHMETRICSFLOAT',
'PGLYPHMETRICSFLOAT', 'LPGLYPHMETRICSFLOAT', 'WGL_FONT_LINES',
'WGL_FONT_POLYGONS', 'wglUseFontOutlinesA', 'wglUseFontOutlinesW',
'LAYERPLANEDESCRIPTOR', 'PLAYERPLANEDESCRIPTOR', 'LPLAYERPLANEDESCRIPTOR',
'LPD_DOUBLEBUFFER', 'LPD_STEREO', 'LPD_SUPPORT_GDI', 'LPD_SUPPORT_OPENGL',
'LPD_SHARE_DEPTH', 'LPD_SHARE_STENCIL', 'LPD_SHARE_ACCUM',
'LPD_SWAP_EXCHANGE', 'LPD_SWAP_COPY', 'LPD_TRANSPARENT', 'LPD_TYPE_RGBA',
'LPD_TYPE_COLORINDEX', 'WGL_SWAP_MAIN_PLANE', 'WGL_SWAP_OVERLAY1',
'WGL_SWAP_OVERLAY2', 'WGL_SWAP_OVERLAY3', 'WGL_SWAP_OVERLAY4',
'WGL_SWAP_OVERLAY5', 'WGL_SWAP_OVERLAY6', 'WGL_SWAP_OVERLAY7',
'WGL_SWAP_OVERLAY8', 'WGL_SWAP_OVERLAY9', 'WGL_SWAP_OVERLAY10',
'WGL_SWAP_OVERLAY11', 'WGL_SWAP_OVERLAY12', 'WGL_SWAP_OVERLAY13',
'WGL_SWAP_OVERLAY14', 'WGL_SWAP_OVERLAY15', 'WGL_SWAP_UNDERLAY1',
'WGL_SWAP_UNDERLAY2', 'WGL_SWAP_UNDERLAY3', 'WGL_SWAP_UNDERLAY4',
'WGL_SWAP_UNDERLAY5', 'WGL_SWAP_UNDERLAY6', 'WGL_SWAP_UNDERLAY7',
'WGL_SWAP_UNDERLAY8', 'WGL_SWAP_UNDERLAY9', 'WGL_SWAP_UNDERLAY10',
'WGL_SWAP_UNDERLAY11', 'WGL_SWAP_UNDERLAY12', 'WGL_SWAP_UNDERLAY13',
'WGL_SWAP_UNDERLAY14', 'WGL_SWAP_UNDERLAY15', 'wglDescribeLayerPlane',
'wglSetLayerPaletteEntries', 'wglGetLayerPaletteEntries',
'wglRealizeLayerPalette', 'wglSwapLayerBuffers', 'WGLSWAP', 'PWGLSWAP',
'LPWGLSWAP', 'WGL_SWAPMULTIPLE_MAX', 'wglSwapMultipleBuffers', 'RECT',
'PRECT', 'NPRECT', 'LPRECT']
# END GENERATED CONTENT (do not edit above this line)




"""Original Numeric module implementation of the OpenGL-ctypes array interfaces

Eventual Goals:
    * Be able to register handlers for a given data-storage mechanism at
        run-time
    * Be able to choose what data-type to use for any given operation where 
        we are getting return values and/or register a default format for
        return values (i.e. tell OpenGL to return ctypes pointers or Numeric 
        arrays or Numarray arrays for glGet* calls)
"""
REGISTRY_NAME = 'numeric'
try:
    import Numeric
except ImportError, err:
    raise ImportError( """No Numeric module present: %s"""%(err))
import operator
from OpenGL.arrays import _numeric

dataPointer = _numeric.dataPointer

from OpenGL import constants, constant
from OpenGL.arrays import formathandler

class NumericHandler( formathandler.FormatHandler ):
    """Numeric-specific data-type handler for OpenGL"""
    HANDLED_TYPES = (Numeric.ArrayType, )
    isOutput = True
    @classmethod
    def from_param( cls, value, typeCode=None ):
        return dataPointer( value )
    dataPointer = staticmethod( dataPointer )
    def voidDataPointer( cls, value ):
        """Given value in a known data-pointer type, return void_p for pointer"""
        return ctypes.c_void_p( self.dataPointer( value ))
    def zeros( self, dims, typeCode ):
        """Return Numeric array of zeros in given size"""
        return Numeric.zeros( dims, GL_TYPE_TO_ARRAY_MAPPING.get(typeCode) or typeCode )
    def arrayToGLType( self, value ):
        """Given a value, guess OpenGL type of the corresponding pointer"""
        typeCode = value.typecode()
        constant = ARRAY_TO_GL_TYPE_MAPPING.get( typeCode )
        if constant is None:
            raise TypeError(
                """Don't know GL type for array of type %r, known types: %s\nvalue:%s"""%(
                    typeCode, ARRAY_TO_GL_TYPE_MAPPING.keys(), value,
                )
            )
        return constant
    def arraySize( self, value, typeCode = None ):
        """Given a data-value, calculate dimensions for the array"""
        try:
            dimValue = value.shape
        except AttributeError, err:
            # XXX it's a list or a tuple, how do we determine dimensions there???
            # for now we'll just punt and convert to an array first...
            value = self.asArray( value, typeCode )
            dimValue = value.shape 
        dims = 1
        for dim in dimValue:
            dims *= dim 
        return dims 
    def asArray( self, value, typeCode=None ):
        """Convert given value to an array value of given typeCode"""
        if value is None:
            return value
        else:
            return self.contiguous( value, typeCode )

    def contiguous( self, source, typeCode=None ):
        """Get contiguous array from source
        
        source -- Numeric Python array (or compatible object)
            for use as the data source.  If this is not a contiguous
            array of the given typeCode, a copy will be made, 
            otherwise will just be returned unchanged.
        typeCode -- optional 1-character typeCode specifier for
            the Numeric.array function.
            
        All gl*Pointer calls should use contiguous arrays, as non-
        contiguous arrays will be re-copied on every rendering pass.
        Although this doesn't raise an error, it does tend to slow
        down rendering.
        """
        typeCode = GL_TYPE_TO_ARRAY_MAPPING.get(  typeCode )
        if isinstance( source, Numeric.ArrayType):
            if source.iscontiguous() and (typeCode is None or typeCode==source.typecode()):
                return source
            else:
                if typeCode is None:
                    typeCode = source.typecode()
                # We have to do astype to avoid errors about unsafe conversions
                # XXX Confirm that this will *always* create a new contiguous array 
                # XXX Allow a way to make this raise an error for performance reasons
                # XXX Guard against wacky conversion types like uint to float, where
                # we really don't want to have the C-level conversion occur.
                return Numeric.array( source.astype( typeCode ), typeCode )
        elif typeCode:
            return Numeric.array( source, typeCode )
        else:
            return Numeric.array( source )
    def unitSize( self, value, typeCode=None ):
        """Determine unit size of an array (if possible)"""
        return value.shape[-1]
    def dimensions( self, value, typeCode=None ):
        """Determine dimensions of the passed array value (if possible)"""
        return value.shape


ARRAY_TO_GL_TYPE_MAPPING = {
    'd': constants.GL_DOUBLE,
    'f': constants.GL_FLOAT,
    'i': constants.GL_INT,
    's': constants.GL_SHORT,
    'c': constants.GL_UNSIGNED_BYTE,
    'b': constants.GL_BYTE,
    'I': constants.GL_UNSIGNED_INT,
}
GL_TYPE_TO_ARRAY_MAPPING = {
    constants.GL_DOUBLE: 'd',
    constants.GL_FLOAT:'f',
    constants.GL_INT: 'i',
    constants.GL_UNSIGNED_INT: 'i',
    constants.GL_UNSIGNED_BYTE: 'c',
    constants.GL_SHORT: 's',
    constants.GL_UNSIGNED_SHORT: 's',
    constants.GL_BYTE: 'b',
}



'''OpenGL extension SUN.global_alpha

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_SUN_global_alpha'
_DEPRECATED = False
GL_GLOBAL_ALPHA_SUN = constant.Constant( 'GL_GLOBAL_ALPHA_SUN', 0x81D9 )
GL_GLOBAL_ALPHA_FACTOR_SUN = constant.Constant( 'GL_GLOBAL_ALPHA_FACTOR_SUN', 0x81DA )
glget.addGLGetConstant( GL_GLOBAL_ALPHA_FACTOR_SUN, (1,) )
glGlobalAlphaFactorbSUN = platform.createExtensionFunction( 
'glGlobalAlphaFactorbSUN',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLbyte,),
doc='glGlobalAlphaFactorbSUN(GLbyte(factor)) -> None',
argNames=('factor',),
deprecated=_DEPRECATED,
)

glGlobalAlphaFactorsSUN = platform.createExtensionFunction( 
'glGlobalAlphaFactorsSUN',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLshort,),
doc='glGlobalAlphaFactorsSUN(GLshort(factor)) -> None',
argNames=('factor',),
deprecated=_DEPRECATED,
)

glGlobalAlphaFactoriSUN = platform.createExtensionFunction( 
'glGlobalAlphaFactoriSUN',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLint,),
doc='glGlobalAlphaFactoriSUN(GLint(factor)) -> None',
argNames=('factor',),
deprecated=_DEPRECATED,
)

glGlobalAlphaFactorfSUN = platform.createExtensionFunction( 
'glGlobalAlphaFactorfSUN',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLfloat,),
doc='glGlobalAlphaFactorfSUN(GLfloat(factor)) -> None',
argNames=('factor',),
deprecated=_DEPRECATED,
)

glGlobalAlphaFactordSUN = platform.createExtensionFunction( 
'glGlobalAlphaFactordSUN',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLdouble,),
doc='glGlobalAlphaFactordSUN(GLdouble(factor)) -> None',
argNames=('factor',),
deprecated=_DEPRECATED,
)

glGlobalAlphaFactorubSUN = platform.createExtensionFunction( 
'glGlobalAlphaFactorubSUN',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLubyte,),
doc='glGlobalAlphaFactorubSUN(GLubyte(factor)) -> None',
argNames=('factor',),
deprecated=_DEPRECATED,
)

glGlobalAlphaFactorusSUN = platform.createExtensionFunction( 
'glGlobalAlphaFactorusSUN',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLushort,),
doc='glGlobalAlphaFactorusSUN(GLushort(factor)) -> None',
argNames=('factor',),
deprecated=_DEPRECATED,
)

glGlobalAlphaFactoruiSUN = platform.createExtensionFunction( 
'glGlobalAlphaFactoruiSUN',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,),
doc='glGlobalAlphaFactoruiSUN(GLuint(factor)) -> None',
argNames=('factor',),
deprecated=_DEPRECATED,
)


def glInitGlobalAlphaSUN():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



#! /usr/bin/env python
"""Test for glGenFramebuffersEXT multiple-call failure reported by Joshua Davis"""
from OpenGL.GL import *
from OpenGL.GLU import *
from OpenGL.GLUT import *
# adapted from http://www.pygame.org/wiki/GLSLExample
from OpenGL.GL.EXT.framebuffer_object import *
def main():
    glutInit(0)
    glutInitDisplayMode(GLUT_RGB|GLUT_DOUBLE|GLUT_DEPTH)
    glutCreateWindow('test')
    framebuffer = glGenFramebuffersEXT(1)
    print type(framebuffer)
    cow = glGenFramebuffersEXT(1)
    print cow
    glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, framebuffer)

if __name__ == "__main__":
    main()
    


'''OpenGL extension EXT.misc_attribute

This module customises the behaviour of the 
OpenGL.raw.GL.EXT.misc_attribute to provide a more 
Python-friendly API

Overview (from the spec)
	
	EXT_misc_attribute extends the list of attribute groups. It provides
	a miscellaneous group, controlled by the MISC_BIT_EXT bit, that contains
	the attribute state of extensions that don't logically fit in any other
	group. 

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/misc_attribute.txt
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions, wrapper
from OpenGL.GL import glget
import ctypes
from OpenGL.raw.GL.EXT.misc_attribute import *
### END AUTOGENERATED SECTION


'''OpenGL extension PGI.misc_hints

This module customises the behaviour of the 
OpenGL.raw.GL.PGI.misc_hints to provide a more 
Python-friendly API

Overview (from the spec)
	
	The extension allows the app to give various hints regarding 
	desired level of conformance, performance, features, etc.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/PGI/misc_hints.txt
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions, wrapper
from OpenGL.GL import glget
import ctypes
from OpenGL.raw.GL.PGI.misc_hints import *
### END AUTOGENERATED SECTION


'''OpenGL extension EXT.fog_coord

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_fog_coord'
_DEPRECATED = False
GL_FOG_COORDINATE_SOURCE_EXT = constant.Constant( 'GL_FOG_COORDINATE_SOURCE_EXT', 0x8450 )
GL_FOG_COORDINATE_EXT = constant.Constant( 'GL_FOG_COORDINATE_EXT', 0x8451 )
GL_FRAGMENT_DEPTH_EXT = constant.Constant( 'GL_FRAGMENT_DEPTH_EXT', 0x8452 )
GL_CURRENT_FOG_COORDINATE_EXT = constant.Constant( 'GL_CURRENT_FOG_COORDINATE_EXT', 0x8453 )
glget.addGLGetConstant( GL_CURRENT_FOG_COORDINATE_EXT, (1,) )
GL_FOG_COORDINATE_ARRAY_TYPE_EXT = constant.Constant( 'GL_FOG_COORDINATE_ARRAY_TYPE_EXT', 0x8454 )
glget.addGLGetConstant( GL_FOG_COORDINATE_ARRAY_TYPE_EXT, (1,) )
GL_FOG_COORDINATE_ARRAY_STRIDE_EXT = constant.Constant( 'GL_FOG_COORDINATE_ARRAY_STRIDE_EXT', 0x8455 )
glget.addGLGetConstant( GL_FOG_COORDINATE_ARRAY_STRIDE_EXT, (1,) )
GL_FOG_COORDINATE_ARRAY_POINTER_EXT = constant.Constant( 'GL_FOG_COORDINATE_ARRAY_POINTER_EXT', 0x8456 )
GL_FOG_COORDINATE_ARRAY_EXT = constant.Constant( 'GL_FOG_COORDINATE_ARRAY_EXT', 0x8457 )
glFogCoordfEXT = platform.createExtensionFunction( 
'glFogCoordfEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLfloat,),
doc='glFogCoordfEXT(GLfloat(coord)) -> None',
argNames=('coord',),
deprecated=_DEPRECATED,
)

glFogCoordfvEXT = platform.createExtensionFunction( 
'glFogCoordfvEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(arrays.GLfloatArray,),
doc='glFogCoordfvEXT(GLfloatArray(coord)) -> None',
argNames=('coord',),
deprecated=_DEPRECATED,
)

glFogCoorddEXT = platform.createExtensionFunction( 
'glFogCoorddEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLdouble,),
doc='glFogCoorddEXT(GLdouble(coord)) -> None',
argNames=('coord',),
deprecated=_DEPRECATED,
)

glFogCoorddvEXT = platform.createExtensionFunction( 
'glFogCoorddvEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(arrays.GLdoubleArray,),
doc='glFogCoorddvEXT(GLdoubleArray(coord)) -> None',
argNames=('coord',),
deprecated=_DEPRECATED,
)

glFogCoordPointerEXT = platform.createExtensionFunction( 
'glFogCoordPointerEXT',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLsizei,ctypes.c_void_p,),
doc='glFogCoordPointerEXT(GLenum(type), GLsizei(stride), c_void_p(pointer)) -> None',
argNames=('type','stride','pointer',),
deprecated=_DEPRECATED,
)


def glInitFogCoordEXT():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



"""ctypes sized data-arrays as a data-formatmechanism

XXX we have to use _ctypes.Array as the type descriminator,
would be nice to have it available from the public module
"""
REGISTRY_NAME = 'ctypesarrays'
import ctypes, _ctypes

from OpenGL import constants, constant
from OpenGL.arrays import formathandler
import operator

class CtypesArrayHandler( formathandler.FormatHandler ):
    """Ctypes Array-type-specific data-type handler for OpenGL"""
    @classmethod
    def from_param( cls, value, typeCode=None ):
        return ctypes.byref( value )
    dataPointer = staticmethod( ctypes.addressof )
    HANDLED_TYPES = (_ctypes.Array, )
    isOutput = True
    def voidDataPointer( cls, value ):
        """Given value in a known data-pointer type, return void_p for pointer"""
        return ctypes.byref( value )
    def zeros( self, dims, typeCode ):
        """Return Numpy array of zeros in given size"""
        type = GL_TYPE_TO_ARRAY_MAPPING[ typeCode ]
        for dim in dims:
            type *= dim 
        return type() # should expicitly set to 0s
    def ones( self, dims, typeCode='d' ):
        """Return numpy array of ones in given size"""
        raise NotImplementedError( """Haven't got a good ones implementation yet""" )
##		type = GL_TYPE_TO_ARRAY_MAPPING[ typeCode ]
##		for dim in dims:
##			type *= dim 
##		return type() # should expicitly set to 0s
    def arrayToGLType( self, value ):
        """Given a value, guess OpenGL type of the corresponding pointer"""
        result = ARRAY_TO_GL_TYPE_MAPPING.get( value._type_ )
        if result is not None:
            return result
        raise TypeError(
            """Don't know GL type for array of type %r, known types: %s\nvalue:%s"""%(
                value._type_, ARRAY_TO_GL_TYPE_MAPPING.keys(), value,
            )
        )
    def arraySize( self, value, typeCode = None ):
        """Given a data-value, calculate dimensions for the array"""
        try:
            return value.__class__.__component_count__
        except AttributeError, err:
            dims = 1
            for length in self.dims( value ):
                dims *= length
            value.__class__.__component_count__ = dims
            return dims 
    def arrayByteCount( self, value, typeCode = None ):
        """Given a data-value, calculate number of bytes required to represent"""
        return ctypes.sizeof( value )
    def types( self, value ):
        """Produce iterable producing all composite types"""
        dimObject = value
        while dimObject is not None:
            yield dimObject
            dimObject = getattr( dimObject, '_type_', None )
            if isinstance( dimObject, (str,unicode)):
                dimObject = None 
    def dims( self, value ):
        """Produce iterable of all dimensions"""
        try:
            return value.__class__.__dimensions__
        except AttributeError, err:
            dimensions = []
            for base in self.types( value ):
                length = getattr( base, '_length_', None)
                if length is not None:
                    dimensions.append( length )
            dimensions = tuple( dimensions )
            value.__class__.__dimensions__  = dimensions
            return dimensions
    def asArray( self, value, typeCode=None ):
        """Convert given value to an array value of given typeCode"""
        return value
    def unitSize( self, value, typeCode=None ):
        """Determine unit size of an array (if possible)"""
        try:
            return value.__class__.__min_dimension__
        except AttributeError, err:
            dim = self.dims( value )[-1]
            value.__class__.__min_dimension__ = dim
            return dim
    def dimensions( self, value, typeCode=None ):
        """Determine dimensions of the passed array value (if possible)"""
        return tuple( self.dims(value) )


ARRAY_TO_GL_TYPE_MAPPING = {
    constants.GLdouble: constants.GL_DOUBLE,
    constants.GLfloat: constants.GL_FLOAT,
    constants.GLint: constants.GL_INT,
    constants.GLuint: constants.GL_UNSIGNED_INT,
    constants.GLshort: constants.GL_SHORT,
    constants.GLushort: constants.GL_UNSIGNED_SHORT,
        
    constants.GLchar: constants.GL_CHAR,
    constants.GLbyte: constants.GL_BYTE,
    constants.GLubyte: constants.GL_UNSIGNED_BYTE,
}
GL_TYPE_TO_ARRAY_MAPPING = {
    constants.GL_DOUBLE: constants.GLdouble,
    constants.GL_FLOAT: constants.GLfloat,
    constants.GL_INT: constants.GLint,
    constants.GL_UNSIGNED_INT: constants.GLuint,
    constants.GL_SHORT: constants.GLshort,
    constants.GL_UNSIGNED_SHORT: constants.GLushort,
        
    constants.GL_CHAR: constants.GLchar,
    constants.GL_BYTE: constants.GLbyte,
    constants.GL_UNSIGNED_BYTE: constants.GLubyte,
}


"""Abstraction for the platform-specific code in PyOpenGL

Each supported platform has a module which provides the
specific functionality required to support the base OpenGL 
functionality on that platform.  These modules are 
registered using plugins in the:

    OpenGL.plugin.PlatformPlugin

objects.  To support a new platform you'll need to create
a new PlatformPlugin instance *before* you import 
OpenGL.platform .  Once you have a working platform 
module, please consider contributing it back to the project.

See baseplatform.BasePlatform for the core functionality 
of a platform implementation.  See the various platform 
specific modules for examples to use when porting.
"""
import os, sys
from OpenGL.plugins import PlatformPlugin

def _load( ):
    """Load the os.name plugin for the platform functionality"""
    
    key = (sys.platform,os.name)
    plugin  = PlatformPlugin.match( key )
    plugin_class = plugin.load()
    plugin.loaded = True
    # create instance of this platform implementation
    plugin = plugin_class()

    # install into the platform module's namespace now
    plugin.install(globals())
    return plugin

_load()


'''OpenGL extension VERSION.GL_1_4_DEPRECATED

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_VERSION_GL_1_4'
_DEPRECATED = True
GL_POINT_SIZE_MIN = constant.Constant( 'GL_POINT_SIZE_MIN', 0x8126 )
GL_POINT_SIZE_MAX = constant.Constant( 'GL_POINT_SIZE_MAX', 0x8127 )
GL_POINT_DISTANCE_ATTENUATION = constant.Constant( 'GL_POINT_DISTANCE_ATTENUATION', 0x8129 )
GL_GENERATE_MIPMAP = constant.Constant( 'GL_GENERATE_MIPMAP', 0x8191 )
GL_GENERATE_MIPMAP_HINT = constant.Constant( 'GL_GENERATE_MIPMAP_HINT', 0x8192 )
GL_FOG_COORDINATE_SOURCE = constant.Constant( 'GL_FOG_COORDINATE_SOURCE', 0x8450 )
GL_FOG_COORDINATE = constant.Constant( 'GL_FOG_COORDINATE', 0x8451 )
GL_FRAGMENT_DEPTH = constant.Constant( 'GL_FRAGMENT_DEPTH', 0x8452 )
GL_CURRENT_FOG_COORDINATE = constant.Constant( 'GL_CURRENT_FOG_COORDINATE', 0x8453 )
GL_FOG_COORDINATE_ARRAY_TYPE = constant.Constant( 'GL_FOG_COORDINATE_ARRAY_TYPE', 0x8454 )
GL_FOG_COORDINATE_ARRAY_STRIDE = constant.Constant( 'GL_FOG_COORDINATE_ARRAY_STRIDE', 0x8455 )
GL_FOG_COORDINATE_ARRAY_POINTER = constant.Constant( 'GL_FOG_COORDINATE_ARRAY_POINTER', 0x8456 )
GL_FOG_COORDINATE_ARRAY = constant.Constant( 'GL_FOG_COORDINATE_ARRAY', 0x8457 )
GL_COLOR_SUM = constant.Constant( 'GL_COLOR_SUM', 0x8458 )
GL_CURRENT_SECONDARY_COLOR = constant.Constant( 'GL_CURRENT_SECONDARY_COLOR', 0x8459 )
GL_SECONDARY_COLOR_ARRAY_SIZE = constant.Constant( 'GL_SECONDARY_COLOR_ARRAY_SIZE', 0x845A )
GL_SECONDARY_COLOR_ARRAY_TYPE = constant.Constant( 'GL_SECONDARY_COLOR_ARRAY_TYPE', 0x845B )
GL_SECONDARY_COLOR_ARRAY_STRIDE = constant.Constant( 'GL_SECONDARY_COLOR_ARRAY_STRIDE', 0x845C )
GL_SECONDARY_COLOR_ARRAY_POINTER = constant.Constant( 'GL_SECONDARY_COLOR_ARRAY_POINTER', 0x845D )
GL_SECONDARY_COLOR_ARRAY = constant.Constant( 'GL_SECONDARY_COLOR_ARRAY', 0x845E )
GL_TEXTURE_FILTER_CONTROL = constant.Constant( 'GL_TEXTURE_FILTER_CONTROL', 0x8500 )
GL_DEPTH_TEXTURE_MODE = constant.Constant( 'GL_DEPTH_TEXTURE_MODE', 0x884B )
GL_COMPARE_R_TO_TEXTURE = constant.Constant( 'GL_COMPARE_R_TO_TEXTURE', 0x884E )
glFogCoordf = platform.createExtensionFunction( 
'glFogCoordf',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLfloat,),
doc='glFogCoordf(GLfloat(coord)) -> None',
argNames=('coord',),
deprecated=_DEPRECATED,
)

glFogCoordfv = platform.createExtensionFunction( 
'glFogCoordfv',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(arrays.GLfloatArray,),
doc='glFogCoordfv(GLfloatArray(coord)) -> None',
argNames=('coord',),
deprecated=_DEPRECATED,
)

glFogCoordd = platform.createExtensionFunction( 
'glFogCoordd',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLdouble,),
doc='glFogCoordd(GLdouble(coord)) -> None',
argNames=('coord',),
deprecated=_DEPRECATED,
)

glFogCoorddv = platform.createExtensionFunction( 
'glFogCoorddv',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(arrays.GLdoubleArray,),
doc='glFogCoorddv(GLdoubleArray(coord)) -> None',
argNames=('coord',),
deprecated=_DEPRECATED,
)

glFogCoordPointer = platform.createExtensionFunction( 
'glFogCoordPointer',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLsizei,ctypes.c_void_p,),
doc='glFogCoordPointer(GLenum(type), GLsizei(stride), c_void_p(pointer)) -> None',
argNames=('type','stride','pointer',),
deprecated=_DEPRECATED,
)

glSecondaryColor3b = platform.createExtensionFunction( 
'glSecondaryColor3b',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLbyte,constants.GLbyte,constants.GLbyte,),
doc='glSecondaryColor3b(GLbyte(red), GLbyte(green), GLbyte(blue)) -> None',
argNames=('red','green','blue',),
deprecated=_DEPRECATED,
)

glSecondaryColor3bv = platform.createExtensionFunction( 
'glSecondaryColor3bv',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(arrays.GLbyteArray,),
doc='glSecondaryColor3bv(GLbyteArray(v)) -> None',
argNames=('v',),
deprecated=_DEPRECATED,
)

glSecondaryColor3d = platform.createExtensionFunction( 
'glSecondaryColor3d',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLdouble,constants.GLdouble,constants.GLdouble,),
doc='glSecondaryColor3d(GLdouble(red), GLdouble(green), GLdouble(blue)) -> None',
argNames=('red','green','blue',),
deprecated=_DEPRECATED,
)

glSecondaryColor3dv = platform.createExtensionFunction( 
'glSecondaryColor3dv',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(arrays.GLdoubleArray,),
doc='glSecondaryColor3dv(GLdoubleArray(v)) -> None',
argNames=('v',),
deprecated=_DEPRECATED,
)

glSecondaryColor3f = platform.createExtensionFunction( 
'glSecondaryColor3f',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLfloat,constants.GLfloat,constants.GLfloat,),
doc='glSecondaryColor3f(GLfloat(red), GLfloat(green), GLfloat(blue)) -> None',
argNames=('red','green','blue',),
deprecated=_DEPRECATED,
)

glSecondaryColor3fv = platform.createExtensionFunction( 
'glSecondaryColor3fv',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(arrays.GLfloatArray,),
doc='glSecondaryColor3fv(GLfloatArray(v)) -> None',
argNames=('v',),
deprecated=_DEPRECATED,
)

glSecondaryColor3i = platform.createExtensionFunction( 
'glSecondaryColor3i',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLint,constants.GLint,constants.GLint,),
doc='glSecondaryColor3i(GLint(red), GLint(green), GLint(blue)) -> None',
argNames=('red','green','blue',),
deprecated=_DEPRECATED,
)

glSecondaryColor3iv = platform.createExtensionFunction( 
'glSecondaryColor3iv',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(arrays.GLintArray,),
doc='glSecondaryColor3iv(GLintArray(v)) -> None',
argNames=('v',),
deprecated=_DEPRECATED,
)

glSecondaryColor3s = platform.createExtensionFunction( 
'glSecondaryColor3s',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLshort,constants.GLshort,constants.GLshort,),
doc='glSecondaryColor3s(GLshort(red), GLshort(green), GLshort(blue)) -> None',
argNames=('red','green','blue',),
deprecated=_DEPRECATED,
)

glSecondaryColor3sv = platform.createExtensionFunction( 
'glSecondaryColor3sv',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(arrays.GLshortArray,),
doc='glSecondaryColor3sv(GLshortArray(v)) -> None',
argNames=('v',),
deprecated=_DEPRECATED,
)

glSecondaryColor3ub = platform.createExtensionFunction( 
'glSecondaryColor3ub',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLubyte,constants.GLubyte,constants.GLubyte,),
doc='glSecondaryColor3ub(GLubyte(red), GLubyte(green), GLubyte(blue)) -> None',
argNames=('red','green','blue',),
deprecated=_DEPRECATED,
)

glSecondaryColor3ubv = platform.createExtensionFunction( 
'glSecondaryColor3ubv',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(arrays.GLubyteArray,),
doc='glSecondaryColor3ubv(GLubyteArray(v)) -> None',
argNames=('v',),
deprecated=_DEPRECATED,
)

glSecondaryColor3ui = platform.createExtensionFunction( 
'glSecondaryColor3ui',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLuint,constants.GLuint,),
doc='glSecondaryColor3ui(GLuint(red), GLuint(green), GLuint(blue)) -> None',
argNames=('red','green','blue',),
deprecated=_DEPRECATED,
)

glSecondaryColor3uiv = platform.createExtensionFunction( 
'glSecondaryColor3uiv',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(arrays.GLuintArray,),
doc='glSecondaryColor3uiv(GLuintArray(v)) -> None',
argNames=('v',),
deprecated=_DEPRECATED,
)

glSecondaryColor3us = platform.createExtensionFunction( 
'glSecondaryColor3us',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLushort,constants.GLushort,constants.GLushort,),
doc='glSecondaryColor3us(GLushort(red), GLushort(green), GLushort(blue)) -> None',
argNames=('red','green','blue',),
deprecated=_DEPRECATED,
)

glSecondaryColor3usv = platform.createExtensionFunction( 
'glSecondaryColor3usv',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(arrays.GLushortArray,),
doc='glSecondaryColor3usv(GLushortArray(v)) -> None',
argNames=('v',),
deprecated=_DEPRECATED,
)

glSecondaryColorPointer = platform.createExtensionFunction( 
'glSecondaryColorPointer',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLint,constants.GLenum,constants.GLsizei,ctypes.c_void_p,),
doc='glSecondaryColorPointer(GLint(size), GLenum(type), GLsizei(stride), c_void_p(pointer)) -> None',
argNames=('size','type','stride','pointer',),
deprecated=_DEPRECATED,
)

glWindowPos2d = platform.createExtensionFunction( 
'glWindowPos2d',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLdouble,constants.GLdouble,),
doc='glWindowPos2d(GLdouble(x), GLdouble(y)) -> None',
argNames=('x','y',),
deprecated=_DEPRECATED,
)

glWindowPos2dv = platform.createExtensionFunction( 
'glWindowPos2dv',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(arrays.GLdoubleArray,),
doc='glWindowPos2dv(GLdoubleArray(v)) -> None',
argNames=('v',),
deprecated=_DEPRECATED,
)

glWindowPos2f = platform.createExtensionFunction( 
'glWindowPos2f',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLfloat,constants.GLfloat,),
doc='glWindowPos2f(GLfloat(x), GLfloat(y)) -> None',
argNames=('x','y',),
deprecated=_DEPRECATED,
)

glWindowPos2fv = platform.createExtensionFunction( 
'glWindowPos2fv',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(arrays.GLfloatArray,),
doc='glWindowPos2fv(GLfloatArray(v)) -> None',
argNames=('v',),
deprecated=_DEPRECATED,
)

glWindowPos2i = platform.createExtensionFunction( 
'glWindowPos2i',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLint,constants.GLint,),
doc='glWindowPos2i(GLint(x), GLint(y)) -> None',
argNames=('x','y',),
deprecated=_DEPRECATED,
)

glWindowPos2iv = platform.createExtensionFunction( 
'glWindowPos2iv',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(arrays.GLintArray,),
doc='glWindowPos2iv(GLintArray(v)) -> None',
argNames=('v',),
deprecated=_DEPRECATED,
)

glWindowPos2s = platform.createExtensionFunction( 
'glWindowPos2s',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLshort,constants.GLshort,),
doc='glWindowPos2s(GLshort(x), GLshort(y)) -> None',
argNames=('x','y',),
deprecated=_DEPRECATED,
)

glWindowPos2sv = platform.createExtensionFunction( 
'glWindowPos2sv',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(arrays.GLshortArray,),
doc='glWindowPos2sv(GLshortArray(v)) -> None',
argNames=('v',),
deprecated=_DEPRECATED,
)

glWindowPos3d = platform.createExtensionFunction( 
'glWindowPos3d',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLdouble,constants.GLdouble,constants.GLdouble,),
doc='glWindowPos3d(GLdouble(x), GLdouble(y), GLdouble(z)) -> None',
argNames=('x','y','z',),
deprecated=_DEPRECATED,
)

glWindowPos3dv = platform.createExtensionFunction( 
'glWindowPos3dv',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(arrays.GLdoubleArray,),
doc='glWindowPos3dv(GLdoubleArray(v)) -> None',
argNames=('v',),
deprecated=_DEPRECATED,
)

glWindowPos3f = platform.createExtensionFunction( 
'glWindowPos3f',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLfloat,constants.GLfloat,constants.GLfloat,),
doc='glWindowPos3f(GLfloat(x), GLfloat(y), GLfloat(z)) -> None',
argNames=('x','y','z',),
deprecated=_DEPRECATED,
)

glWindowPos3fv = platform.createExtensionFunction( 
'glWindowPos3fv',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(arrays.GLfloatArray,),
doc='glWindowPos3fv(GLfloatArray(v)) -> None',
argNames=('v',),
deprecated=_DEPRECATED,
)

glWindowPos3i = platform.createExtensionFunction( 
'glWindowPos3i',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLint,constants.GLint,constants.GLint,),
doc='glWindowPos3i(GLint(x), GLint(y), GLint(z)) -> None',
argNames=('x','y','z',),
deprecated=_DEPRECATED,
)

glWindowPos3iv = platform.createExtensionFunction( 
'glWindowPos3iv',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(arrays.GLintArray,),
doc='glWindowPos3iv(GLintArray(v)) -> None',
argNames=('v',),
deprecated=_DEPRECATED,
)

glWindowPos3s = platform.createExtensionFunction( 
'glWindowPos3s',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLshort,constants.GLshort,constants.GLshort,),
doc='glWindowPos3s(GLshort(x), GLshort(y), GLshort(z)) -> None',
argNames=('x','y','z',),
deprecated=_DEPRECATED,
)

glWindowPos3sv = platform.createExtensionFunction( 
'glWindowPos3sv',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(arrays.GLshortArray,),
doc='glWindowPos3sv(GLshortArray(v)) -> None',
argNames=('v',),
deprecated=_DEPRECATED,
)




'''OpenGL extension EXT.texture_shared_exponent

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_texture_shared_exponent'
_DEPRECATED = False
GL_RGB9_E5_EXT = constant.Constant( 'GL_RGB9_E5_EXT', 0x8C3D )
GL_UNSIGNED_INT_5_9_9_9_REV_EXT = constant.Constant( 'GL_UNSIGNED_INT_5_9_9_9_REV_EXT', 0x8C3E )
GL_TEXTURE_SHARED_SIZE_EXT = constant.Constant( 'GL_TEXTURE_SHARED_SIZE_EXT', 0x8C3F )


def glInitTextureSharedExponentEXT():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension EXT.index_array_formats

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_index_array_formats'
_DEPRECATED = False
GL_IUI_V2F_EXT = constant.Constant( 'GL_IUI_V2F_EXT', 0x81AD )
GL_IUI_V3F_EXT = constant.Constant( 'GL_IUI_V3F_EXT', 0x81AE )
GL_IUI_N3F_V2F_EXT = constant.Constant( 'GL_IUI_N3F_V2F_EXT', 0x81AF )
GL_IUI_N3F_V3F_EXT = constant.Constant( 'GL_IUI_N3F_V3F_EXT', 0x81B0 )
GL_T2F_IUI_V2F_EXT = constant.Constant( 'GL_T2F_IUI_V2F_EXT', 0x81B1 )
GL_T2F_IUI_V3F_EXT = constant.Constant( 'GL_T2F_IUI_V3F_EXT', 0x81B2 )
GL_T2F_IUI_N3F_V2F_EXT = constant.Constant( 'GL_T2F_IUI_N3F_V2F_EXT', 0x81B3 )
GL_T2F_IUI_N3F_V3F_EXT = constant.Constant( 'GL_T2F_IUI_N3F_V3F_EXT', 0x81B4 )


def glInitIndexArrayFormatsEXT():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



"""Test simple functions (i.e. no pointers involved)"""
from OpenGL.GL import *
from OpenGL.GLU import *
from OpenGL.GLUT import *
from OpenGL.constants import GLdouble
from OpenGL.arrays import arraydatatype
from OpenGL.constants import GLdouble
handler = arraydatatype.ArrayDatatype.getHandler( GLdouble * 16 )
handler.registerReturn( )

import time
start = time.time()

window = None

def display():
    glutSetWindow(window);
    glClearColor (0.0, 0.0, (time.time()%1.0)/1.0, 0.0)
    glClear (GL_COLOR_BUFFER_BIT)
    glMatrixMode(GL_PROJECTION);
    # For some reason the GL_PROJECTION_MATRIX is overflowing with a single push!
    # glPushMatrix()
    matrix = glGetDoublev( GL_PROJECTION_MATRIX)
    print 'matrix', type(matrix), matrix
    glutSwapBuffers()

size = (250,250)

def reshape( *args ):
    global size 
    size = args
    glViewport( *( (0,0)+args) )
    display()

def ontimer( *args ):
    print 'timer', args, '@time', time.time()-start
    glutTimerFunc( 1000, ontimer, 24 )

def idle():
    delta = time.time()-start
    if delta < 10:
        global size 
        x,y = size 
        if delta < 5:
            change = +1
        else:
            change = -1
        x = x-change
        y = y+change
        if x < 1:
            x = 1
        if y < 1:
            y = 1
        glutReshapeWindow( x, y )
        size = (x,y)
        glutSetWindow(window)
        glutPostRedisplay()
    else:
        glutDestroyWindow( window )
        print 'window destroyed'
        import sys
        sys.exit( 0 )

def printFunction( name ):
    def onevent( *args ):
        print '%s -> %s'%(name, ", ".join( [str(a) for a in args ]))
    return onevent



if __name__ == "__main__":
    import sys
    newArgv = glutInit(sys.argv)
    print 'newArguments', newArgv
    glutInitDisplayMode( GLUT_DOUBLE | GLUT_RGB )
    glutInitWindowSize(250, 250)
    glutInitWindowPosition(100, 100)
    window = glutCreateWindow("hello")
    print 'window', repr(window)
    glutDisplayFunc(display)
    glutReshapeFunc(reshape)
    glutMouseFunc(printFunction( 'Mouse' ))
    glutEntryFunc(printFunction( 'Entry' ))
    glutKeyboardFunc( printFunction( 'Keyboard' ))
    glutKeyboardUpFunc( printFunction( 'KeyboardUp' ))
    glutMotionFunc( printFunction( 'Motion' ))
    glutPassiveMotionFunc( printFunction( 'PassiveMotion' ))
    glutVisibilityFunc( printFunction( 'Visibility' ))
    glutWindowStatusFunc( printFunction( 'WindowStatus' ))
    glutSpecialFunc( printFunction( 'Special' ))
    glutSpecialUpFunc( printFunction( 'SpecialUp' ))
    glutTimerFunc( 1000, ontimer, 23 )
    
    glutIdleFunc( idle )
    glutMainLoop()



#! /usr/bin/env python
"""Script to download OpenGL extensions header and produce wrappers

This script is the mechanism that allows for automatically
wrapping new extensions with basic ctypes-level functionality.
It doesn't do everything, but it approximates the same level
of access as GLEW or pyglet will give you.

The script also downloads the extension specification into .txt
files that sit next to the same-named .py files, the purpose of
this is to allow developers to grep through the source files 
looking for common features, such as references to the glGet*
tables.

glGetBoolean/glGetInteger/glGetFloat/glGetDouble 
    
    A CSV file in this directory controls registration of 
    constants for use with glGet, the format of the file is
    a tab-delimited format with the constants in the first 
    column.  Records are as follows:
    
    For no registration (e.g. when you need a custom function):

        CONSTANT
    
    For a static size:
    
        CONSTANT	(1,)
    
    For dynamic dictionary-based lookup of size based on another
    parameter:
    
        CONSTANT	pname	LOOKUPCONSTANT	(1,)	LOOKUPCONSTANT	(4,)	...

    Note that only constants which appear in a downloadable 
    specification (.txt file) under a New Tokens header with a note
    that they can be passed to glGetBoolean will be so registered.
"""
import urllib, os, sys, re, string, traceback, logging, textwrap
EXTENSION_HEADER_SOURCE = 'http://www.opengl.org/registry/api/glext.h'
#ROOT_EXTENSION_SOURCE = 'http://oss.sgi.com/projects/ogl-sample/registry/'
ROOT_EXTENSION_SOURCE = 'http://www.opengl.org/registry/specs/'
AUTOGENERATION_SENTINEL = """### DO NOT EDIT above the line "END AUTOGENERATED SECTION" below!"""
AUTOGENERATION_SENTINEL_END = """### END AUTOGENERATED SECTION"""
if not os.path.isdir( os.path.join('..','OpenGL','GL') ):
    raise RuntimeError( """Only run this script with the src directory as the current working directory""" )

log = logging.getLogger( 'autogen' )

def download( url ):
    """Download the given url, informing the user of what we're doing"""
    sys.stderr.write( 'Download: %r\n'%(url,))
    file = urllib.urlopen( url )
    return file.read()


registry = {}

def nameToPath( name ):
    return os.path.join( * name.split( '_',2 ) )
def nameToPathMinusGL( name ):
    return "/".join( name.split( '_',2 )[1:] )

def indent( text, indent='\t' ):
    return "\n".join([
        '%s%s'%(indent,line) 
        for line in text.splitlines()
    ])


class Helper( object ):
    root = ROOT_EXTENSION_SOURCE
    def __getitem__( self, key ):
        item = getattr( self, key, None )
        if item is None:
            raise KeyError( key )
        if callable( item ):
            return item()
        else:
            return item

class Function( Helper ):
    def __init__( self, returnType, name, signature, dll='platform.GL' ):
        """Parse definition into our various elements"""
        self.returnType = self.parseReturnType(returnType)
        self.name = name
        try:
            self.argTypes, self.argNames = self.parseArguments( signature )
        except Exception, err:
            log.error( """Error parsing arguments for %s %s: %s""", name, signature, err )
            self.argTypes, self.argNames = (), ()
##		self.pysignature = '(%s)'%(
##			", ".join([
##				item.split()[-1].strip('*')
##				for item in signature[1:-1].split( ',' )
##				if item.strip().strip('*')
##			])
##		)
        self.dll = dll
    findName = re.compile( '[a-zA-z0-9]*$' )
    def parseReturnType( self, returnType ):
        return self.cTypeToPyType( returnType )
    def parseArguments( self, signature ):
        """Parse a C argument-type declaration into a ctypes-style argTypes and argNames"""
        signature = signature.strip()[1:-1]
        # first and easiest case is a void call...
        if not signature.strip() or signature.strip() == 'void':
            return (), ()
        types, names = [], []
        for item in signature.split( ',' ):
            item = item.strip()
            nameMatch = self.findName.search( item )
            if not nameMatch:
                raise ValueError( item )
            name = nameMatch.group(0)
            rest = item[:nameMatch.start(0)].strip()
            types.append( self.cTypeToPyType( rest ) )
            names.append( name )
        return types, names
    def cTypeToPyType( self, base ):
        """Given a C declared type for an argument/return type, get Python/ctypes version"""
        base = base.strip()
        if base.endswith( 'const' ):
            return self.cTypeToPyType( base[:-5] )
        elif base.startswith( 'const' ):
            return self.cTypeToPyType( base[5:] )
        elif base.endswith( '*' ):
            new = self.cTypeToPyType( base[:-1] )
            if new == 'constants.GLvoid':
                return 'ctypes.c_void_p'
            elif self.CTYPE_TO_ARRAY_TYPE.has_key( new ):
                return 'arrays.%s'%(self.CTYPE_TO_ARRAY_TYPE[new])
            elif new in ( 'arrays.GLcharArray','arrays.GLcharARBArray'):
                # can't have a pointer to these...
                return 'ctypes.POINTER( ctypes.POINTER( constants.GLchar ))'
            elif new in ( 'constants.GLcharARB',):
                return 'ctypes.POINTER( ctypes.c_char_p )'
            else:
                log.warn( 'Unconverted pointer type in %s: %r', self.name, new )
                return 'ctypes.POINTER(%s)'%(new)
        else:
            return 'constants.%s'%(base,)
    def errorReturn( self ):
        return '0'
    def declaration( self ):
        """Produce a declaration for this function in ctypes format"""
        dll = self.dll
        returnType = self.returnType
        if self.argTypes:
            argTypes = '(%s,)'%( ','.join(self.argTypes))
        else:
            argTypes = '()'
        if self.argNames:
            argNames = '(%s,)'%( ','.join([repr(x) for x in self.argNames]))
        else:
            argNames = '()'
        arguments = ', '.join([
            '%(type)s(%(name)s)'%locals()
            for (type,name) in [
                (type.split('.',1)[1],name)
                for type,name in zip( self.argTypes,self.argNames )
            ]
        ])
        name = self.name 
        if returnType.strip() in ('constants.GLvoid', 'constants.void'):
            returnType = pyReturn = 'None'
        else:
            pyReturn = self.returnType
        log.info( 'returnType %s -> %s', self.returnType, pyReturn )
        doc = '%(name)s(%(arguments)s) -> %(pyReturn)s'%locals()
        return self.TEMPLATE%locals()
    TEMPLATE = """%(name)s = platform.createExtensionFunction( 
%(name)r,dll=%(dll)s,
extension=EXTENSION_NAME,
resultType=%(returnType)s, 
argTypes=%(argTypes)s,
doc=%(doc)r,
argNames=%(argNames)s,
deprecated=_DEPRECATED,
)
"""
    CTYPE_TO_ARRAY_TYPE = {
        'constants.GLfloat': 'GLfloatArray',
        'constants.float': 'GLfloatArray',
        'constants.GLclampf': 'GLclampfArray',
        'constants.GLdouble': 'GLdoubleArray',
        'constants.double': 'GLdoubleArray',
        'constants.int': 'GLintArray',
        'constants.GLint': 'GLintArray',
        'constants.GLuint': 'GLuintArray',
        'constants.unsigned int':'GLuintArray',
        'constants.unsigned char': 'GLbyteArray',
        'constants.uint': 'GLuintArray',
        'constants.GLshort': 'GLshortArray',
        'constants.GLushort': 'GLushortArray',
        'constants.short unsigned int':'GLushortArray',
        'constants.GLubyte': 'GLubyteArray',
        'constants.GLbool': 'GLbooleanArray',
        'constants.GLboolean': 'GLbooleanArray',
        'arrays.GLbooleanArray': 'GLbooleanArray',
        'constants.GLbyte': 'GLbyteArray',
        'constants.char': 'GLbyteArray',
        'constants.gleDouble': 'GLdoubleArray',
        'constants.GLchar': 'GLcharArray',
        'constants.GLcharARB': 'GLcharARBArray',
        'constants.GLhalfNV': 'GLushortArray',
        'constants.GLhandle': 'GLuintArray',
        'constants.GLhandleARB': 'GLuintArray',
        'constants.GLenum': 'GLuintArray',
        # following should all have special sub-classes that enforce dimensions
        'constants.gleDouble * 4': 'GLdoubleArray',
        'constants.gleDouble * 3': 'GLdoubleArray',
        'constants.gleDouble * 2': 'GLdoubleArray',
        'constants.c_float * 3': 'GLfloatArray',
        'constants.gleDouble * 3 * 2': 'GLdoubleArray',
        'constants.GLsizei': 'GLsizeiArray',
        'constants.GLint64': 'GLint64Array',
        'constants.GLint64EXT': 'GLint64Array',
        'constants.GLuint64': 'GLuint64Array',
        'constants.GLuint64EXT': 'GLuint64Array',
    }
    
# Don't know how Tarn got the api_versions, short of manually entering them...
WRAPPER_TEMPLATE = """'''OpenGL extension %(owner)s.%(module)s

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = %(constantModule)r
_DEPRECATED = %(deprecatedFlag)r
%(constants)s
%(declarations)s%(deprecated)s
"""

INIT_TEMPLATE = """
def glInit%(camelModule)s%(owner)s():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )
"""
FINAL_MODULE_TEMPLATE = """'''OpenGL extension %(owner)s.%(module)s

This module customises the behaviour of the 
OpenGL.raw.%(prefix)s.%(owner)s.%(module)s to provide a more 
Python-friendly API

%(overview)sThe official definition of this extension is available here:
%(root)s%(owner)s/%(module)s.txt
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions, wrapper
from OpenGL.GL import glget
import ctypes
from OpenGL.raw.%(prefix)s.%(owner)s.%(module)s import *
"""

class Module( Helper ):
    targetDirectory = os.path.join( '..','OpenGL')
    rawTargetDirectory = os.path.join( '..','OpenGL','raw')
    prefix = 'GL'
    defineFinder = re.compile( r'\#define[ \t]+([a-zA-Z0-9_]+)[ \t]*(0x[0-9a-fA-F]+)' )
    functionFinder = re.compile( r'GLAPI[ \t]+(.*?)[ \t]+APIENTRY[ \t]+([a-zA-Z0-9_]+)[ \t]*\(' )
    signatureFinderTemplate = r'typedef[ \t]+%(returnTypeRE)s[ \t]+\(APIENTRYP[ \t]+PFN%(nameUpper)sPROC\)[ \t]*(\(.*?\))[;]'
    typeDefFinder = re.compile( r'typedef[ \t]+(([a-zA-Z0-9_]+[ \t]*)+);' )
    RAW_MODULE_TEMPLATE = WRAPPER_TEMPLATE + INIT_TEMPLATE
    
    def __init__( self, name, segments, header ):
        log.info( 'name: %r', name )
        if not name.startswith( 'GL_' ):
            name = 'GL_'+name
        self.name = name
        self.segments = segments
        self.header = header
        try:
            self.prefix, self.owner, self.module = name.split('_',2)
            self.sentinelConstant = '%s_%s'%(self.owner,self.module)
            
        except ValueError:
            if name.endswith( 'SGIX' ):
                self.prefix = "GL"
                self.owner = 'SGIX'
                self.module = name[3:-4]
                self.sentinelConstant = '%s%s'%(self.module,self.owner)
            else:
                log.error( """Unable to parse module name: %s""", name )
                raise
        if self.module[0] in string.digits:
            self.module = 'GL_%s'%(self.module,)
        self.camelModule = "".join([x.title() for x in self.module.split('_')])
        self.rawModule = self.module
        
        # XXX need to figure this out better
        self.rawOwner = self.owner
        while self.owner and self.owner[0] in string.digits:
            self.owner = self.owner[1:]
        self.rawPathName = os.path.join( self.rawTargetDirectory, self.prefix, self.owner, self.module+'.py' )
        self.pathName = os.path.join( self.targetDirectory, self.prefix, self.owner, self.module+'.py' )
        
        self.findFunctions()
        self.constantModule = '%(prefix)s_%(owner)s_%(rawModule)s'%self
        if self.rawModule.endswith( '_DEPRECATED' ):
            self.constantModule = self.constantModule[:-len('_DEPRECATED')]
            self.deprecatedFlag = True 
        else:
            self.deprecatedFlag = False
        specification = self.getSpecification()
        self.overview = ''
        if self.header.includeOverviews:
            for title,section in specification.blocks( specification.source ):
                if title.startswith( 'Overview' ):
                    self.overview = 'Overview (from the spec)\n%s\n\n'%(
                        indent( section.replace('\xd4','O').replace('\xd5','O') )
                    )
                    break

    def shouldReplace( self ):
        """Should we replace the given filename?"""
        filename = self.pathName
        if not os.path.isfile(
            filename
        ):
            return True
        else:
            hasLines = 0
            for line in open( filename ):
                if line.strip() == AUTOGENERATION_SENTINEL_END.strip():
                    return True
                hasLines = 1
            if not hasLines:
                return True
        return False


    def findFunctions( self ):
        """Find all of our function definitions"""
        result = []
        for segment in self.segments:
            for match in self.functionFinder.finditer(segment):
                returnType, name = match.groups()
                nameUpper = re.escape(name.upper())
                returnTypeRE = re.escape( returnType )
                signatureFinder = re.compile( self.signatureFinderTemplate%locals() )
                try:
                    signature = signatureFinder.search( segment ).group(1)
                    result.append( Function( returnType, name, signature ))
                except AttributeError:
                    log.warn( 
                        "Couldn't find signature for function %s %s",
                        returnType,name,
                    )
        self.functions = result
    def declarations( self ):
        """
        DECLARE_VOID_EXT(glPointParameterfARB, (GLenum pname, GLfloat param), (pname, param))
        DECLARE_VOID_EXT(glPointParameterfvARB, (GLenum pname, const GLfloat* param), (pname, param))
        """
        result = []
        for function in self.functions:
            result.append( function.declaration() )
        return "\n".join( result )
    def functionNames( self ):
        """
        "glPointParameterfARB",
        "glPointParameterfvARB",
        """
        result = []
        for function in self.functions:
            result.append( '"%s",'%(function.name,))
        return "\n".join(result)
    def swigFunctionDeclarations( self ):
        """
        void glPointParameterfARB(GLenum pname, GLfloat param);
        DOC(glPointParameterfARB, "glPointParameterfARB(pname, param) -> None")

        void glPointParameterfvARB(GLenum pname, const GLfloat* param);
        DOC(glPointParameterfvARB, "glPointParameterfvARB(pname, param) -> None")
        """
        result = []
        for segment in self.segments:
            for match in self.typeDefFinder.finditer( segment ):
                result.append( match.group(0))
        for function in self.functions:
            result.append( '%(returnType)s %(name)s%(signature)s;'%function )
            result.append( 'DOC(%(name)s, "%(name)s%(pysignature)s")'%function )
        return "\n".join( result )
    def constants( self ):
        """Retrieve constants from the segments

        This is, of course, all heuristically done :)
        """
        result = []
        glGets = self.getSpecification().glGetConstants()
        glGetSizes = self.header.glGetSizes
        for segment in self.segments:
            for match in self.defineFinder.finditer( segment ):
                name,value = match.groups()
                value = int(value,0)
                result.append( '%(name)s = constant.Constant( %(name)r, 0x%(value)X )'%locals() )
                if glGets.has_key( name ) or glGetSizes.has_key( name ):
                    size = glGetSizes.get( name, [] )
                    if len(size) == 0: # not yet specified...
                        glGetSizes[ name ] = []
                    elif len(size) == 1: # static size...
                        size = size[0]
                        result.append(
                            """glget.addGLGetConstant( %(name)s, %(size)s )"""%locals()
                        )
                    else:
                        # param name, then (key,value) for rest of elements
                        param = size[0]
                        rest = size[1:]
                        set = {}
                        while rest:
                            current = rest[:2]
                            del rest[:2]
                            if len(current) == 2:
                                set[current[0]] = current[1]
                            else:
                                log.warn( 
                                    """Incorrect format for glGet constant %s (unevent set of values)""",
                                    name,
                                )
                        size = '{ %s }'%(
                            ','.join([
                                '%s : %s'%(
                                    key,value 
                                )
                                for (key,value) in set 
                            ])
                        )
                        result.append(
                            """glget.addGLGetConstant( %(name)s, %(size)s, %(param)r )"""%locals()
                        )
        return '\n'.join(result)
    SPEC_EXCEPTIONS = {
        # different URLs... grr...
        '3DFX/multisample': 'http://oss.sgi.com/projects/ogl-sample/registry/3DFX/3dfx_multisample.txt',
        #'EXT/color_matrix': 'http://oss.sgi.com/projects/ogl-sample/registry/SGI/color_matrix.txt',
        #'EXT/texture_cube_map': 'http://oss.sgi.com/projects/ogl-sample/registry/ARB/texture_cube_map.txt',
        'SGIS/fog_function': 'http://oss.sgi.com/projects/ogl-sample/registry/SGIS/fog_func.txt',
    }
    def getSpecification( self ):
        """Retrieve our specification document...
        
        Retrieves the .txt file which defines this specification,
        allowing us to review the document locally in order to provide
        a reasonable wrapping of it...
        """
        specFile = os.path.splitext( self.pathName )[0] + '.txt'
        specURLFragment = nameToPathMinusGL(self.name)
        if self.SPEC_EXCEPTIONS.has_key( specURLFragment ):
            specURL = self.SPEC_EXCEPTIONS[ specURLFragment ]
        else:
            specURL = '%s/%s.txt'%( 
                ROOT_EXTENSION_SOURCE, 
                specURLFragment,
            )
        if not os.path.isfile( specFile ):
            try:
                data = download(specURL)
            except Exception, err:
                log.warn( """Failure downloading specification %s: %s""", specURL, err )
                data = ""
            else:
                try:
                    open(specFile,'w').write( data )
                except IOError, err:
                    pass
        else:
            data = open( specFile ).read()
        if 'Error 404' in data:
            log.info( """Spec 404: %s""", specURL)
            data = ''
        return Specification( data )
    def process( self ):
        """(re)Wrap the given module"""
        # first the raw wrapped API...
        directory = os.path.dirname(self.rawPathName)
        try:
            os.makedirs( directory )
        except os.error:
            pass
        if not os.path.isfile( os.path.join(directory, '__init__.py')):
            open( os.path.join(directory, '__init__.py'),'w').write( 
                '''"""OpenGL Extensions"""'''
            )
        current = ''
        toWrite = self.RAW_MODULE_TEMPLATE % self
        try:
            current = open( self.rawPathName, 'r').read()
        except Exception, err:
            pass 
        if current.strip() != toWrite.strip():
            fh = open( self.rawPathName, 'w')
            fh.write( toWrite )
            fh.close()
        if self.shouldReplace( ):
            # now the final module with any included custom code...
            toWrite = FINAL_MODULE_TEMPLATE % self
            current = ''
            try:
                current = open( self.pathName, 'r').read()
            except Exception, err:
                pass 
            else:
                found = current.rfind( '\n'+AUTOGENERATION_SENTINEL_END )
                if found >= -1:
                    if current[:found].strip() == toWrite.strip():
                        # we aren't going to change anything...
                        return False
                    found += len( '\n' + AUTOGENERATION_SENTINEL_END )
                    current = current[found:]
                else:
                    current = ''
            try:
                fh = open( self.pathName, 'w')
            except IOError, err:
                log.warn( "Unable to create module for %r %s", self.name, err )
                return False
            else:
                fh.write( toWrite )
                fh.write( AUTOGENERATION_SENTINEL_END )
                fh.write( current )
                fh.close()
                return True
        return False
    
    def deprecated( self ):
        """Produce import line for deprecated functions if appropriate"""
        name = self.name + '_DEPRECATED'
        if self.header.registry.get( name ):
            return '''# import legacy entry points to allow checking for bool(entryPoint)
from OpenGL.raw.%(prefix)s.%(owner)s.%(module)s_DEPRECATED import *'''%self
        return ''

class VersionModule( Module ):
    """Module representing an OpenGL version's extension to the spec"""
    targetDirectory = os.path.join( '..','OpenGL')
    rawTargetDirectory = os.path.join( '..','OpenGL','raw')
    prefix = 'GL'
    RAW_MODULE_TEMPLATE = WRAPPER_TEMPLATE
    def getSpecification( self ):
        """Retrieve our specification document...
        
        Retrieves the .txt file which defines this specification,
        allowing us to review the document locally in order to provide
        a reasonable wrapping of it...
        """
        return Specification( '' )
    

class Specification( object ):
    """Parser for parsing OpenGL specifications for interesting information
    
    """
    def __init__( self, source ):
        """Store the source text for the specification"""
        self.source = source
    def blocks( self, data ):
        """Retrieve the set of all blocks"""
        data = data.splitlines()
        title = []
        block = []
        for line in data:
            if line and line.lstrip() == line:
                if block:
                    yield "\n".join(title), textwrap.dedent( "\n".join(block) )
                    title = [ ]
                    block = [ ]
                title.append( line )
            else:
                block.append( line )
        if block:
            yield "\n".join(title), textwrap.dedent( "\n".join(block) )
    def constantBlocks( self ):
        """Retrieve the set of constant blocks"""
        for title,block in self.blocks( self.source ):
            if title and title.startswith( 'New Tokens' ):
                yield block
    def glGetConstants( self ):
        """Retrieve the set of constants which pass to glGet* functions"""
        table = {}
        for block in self.constantBlocks():
            for title, section in self.blocks( block ):
                for possible in (
                    'GetBooleanv','GetIntegerv','<pname> of Get'
                ):
                    if possible in title:
                        for line in section.splitlines():
                            line = line.strip().split()
                            if len(line) == 2:
                                constant,value = line 
                                table['GL_%s'%(constant,)] = value 
                        break
        return table



class Header( object ):
    """Manages the overall header source
    
    registry -- registry of extensions/versions found with the 
        header segments that define them...
    includeOverviews -- if True, include the specification's 
        overviews in the indivdual extensions
    """
    registry = None
    includeOverviews = True
    def getFile( self ):
        """Load or download the source of the glext.h header"""
        if not os.path.isfile( 'glext.h' ):
            data = download( EXTENSION_HEADER_SOURCE )
            open( 'glext.h', 'w').write( data )
        else:
            data = open( 'glext.h' ).read()
        return data
    def getRegistry( self ):
        """Retrieve a parsed registry of extensions/versions
        
        This uses accidents of the header definition to produce
        the results, but the header is fairly consistent...
        
        returns { name: segments} to pass to Module init
        """
        if self.registry:
            return self.registry
        file = self.getFile()
        index = file.find( '#define GL_GLEXT_VERSION' )
        file = file[index:]
        extensions = file.split( '\n#ifndef ' )[1:]
        for item in extensions:
            name, definition = item.split( None, 1 )
            definition = '#ifndef '+item
            registry.setdefault( name, []).append( definition )
        self.registry = registry
        return registry
    def iterModules( self ):
        """Yield each Module( name, segments ) for all extensions
        
        extensions do *not* include the GL core versions...
        """
        items = self.getRegistry().items()
        items.sort()
        for name, segments in items:
            if name in ('APIENTRY','APIENTRYP','GLAPI'):
                continue
            if not name.startswith( 'GL_VERSION' ):
                yield Module( name, segments, header=self )
            else:
                yield VersionModule( name, segments, header=self )
    def iterVersions( self ):
        """Yield each Version( name, segments ) for all versions"""
        items = self.getRegistry().items()
        items.sort()
        for name, segments in items:
            if name.startswith( 'GL_VERSION' ):
                yield Version( name, segments )

    def autoGenerate( self ):
        """Autogenerate all Modules in this header"""
        new = {}
        total = count = 0
        for module in self.iterModules():
            if module.process( ):
                new[module.constantModule] = module
                count += 1
            total += 1
        return total, count
    
    def constantSections( self ):
        """Print the constant sections for all modules"""
        for module in self.iterModules():
            module.getSpecification()
            for constant, value in module.getSpecification().glGetConstants().items():
                #print title
                print constant
    
    glGetSizes = {}
    def loadGLGetSizes( self ):
        """Load manually-generated table of glGet* sizes"""
        table = self.glGetSizes
        try:
            lines = [
                line.split('\t')
                for line in open( 'glgetsizes.csv' ).read().splitlines()
            ]
        except IOError, err:
            pass 
        else:
            for line in lines:
                if line and line[0]:
                    table[line[0].strip('"')] = [
                        v for v in [
                            v.strip('"') for v in line[1:]
                        ]
                        if v
                    ]
    def saveGLGetSizes( self ):
        """Save out sorted list of glGet sizes to disk"""
        items = self.glGetSizes.items()
        items.sort()
        data = "\n".join([
            '%s\t%s'%(
                key,"\t".join(value)
            )
            for (key,value) in items 
        ])
        open( 'glgetsizes.csv','w').write( data )


if __name__ == "__main__":
    logging.basicConfig()
    log.setLevel( logging.WARN )
    header = Header()
    header.loadGLGetSizes()
    total,count = Header().autoGenerate()
    print '%s total %s replaced'%(total,count)
    header.saveGLGetSizes()
    #header.constantSections()



'''OpenGL extension SGIX.pixel_texture

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_SGIX_pixel_texture'
_DEPRECATED = False
GL_PIXEL_TEX_GEN_SGIX = constant.Constant( 'GL_PIXEL_TEX_GEN_SGIX', 0x8139 )
GL_PIXEL_TEX_GEN_MODE_SGIX = constant.Constant( 'GL_PIXEL_TEX_GEN_MODE_SGIX', 0x832B )
glPixelTexGenSGIX = platform.createExtensionFunction( 
'glPixelTexGenSGIX',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,),
doc='glPixelTexGenSGIX(GLenum(mode)) -> None',
argNames=('mode',),
deprecated=_DEPRECATED,
)


def glInitPixelTextureSGIX():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension AMD.performance_monitor

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_AMD_performance_monitor'
_DEPRECATED = False
GL_COUNTER_TYPE_AMD = constant.Constant( 'GL_COUNTER_TYPE_AMD', 0x8BC0 )
GL_COUNTER_RANGE_AMD = constant.Constant( 'GL_COUNTER_RANGE_AMD', 0x8BC1 )
GL_UNSIGNED_INT64_AMD = constant.Constant( 'GL_UNSIGNED_INT64_AMD', 0x8BC2 )
GL_PERCENTAGE_AMD = constant.Constant( 'GL_PERCENTAGE_AMD', 0x8BC3 )
GL_PERFMON_RESULT_AVAILABLE_AMD = constant.Constant( 'GL_PERFMON_RESULT_AVAILABLE_AMD', 0x8BC4 )
GL_PERFMON_RESULT_SIZE_AMD = constant.Constant( 'GL_PERFMON_RESULT_SIZE_AMD', 0x8BC5 )
GL_PERFMON_RESULT_AMD = constant.Constant( 'GL_PERFMON_RESULT_AMD', 0x8BC6 )
glGetPerfMonitorGroupsAMD = platform.createExtensionFunction( 
'glGetPerfMonitorGroupsAMD',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(arrays.GLintArray,constants.GLsizei,arrays.GLuintArray,),
doc='glGetPerfMonitorGroupsAMD(GLintArray(numGroups), GLsizei(groupsSize), GLuintArray(groups)) -> None',
argNames=('numGroups','groupsSize','groups',),
deprecated=_DEPRECATED,
)

glGetPerfMonitorCountersAMD = platform.createExtensionFunction( 
'glGetPerfMonitorCountersAMD',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,arrays.GLintArray,arrays.GLintArray,constants.GLsizei,arrays.GLuintArray,),
doc='glGetPerfMonitorCountersAMD(GLuint(group), GLintArray(numCounters), GLintArray(maxActiveCounters), GLsizei(counterSize), GLuintArray(counters)) -> None',
argNames=('group','numCounters','maxActiveCounters','counterSize','counters',),
deprecated=_DEPRECATED,
)

glGetPerfMonitorGroupStringAMD = platform.createExtensionFunction( 
'glGetPerfMonitorGroupStringAMD',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLsizei,arrays.GLsizeiArray,arrays.GLcharArray,),
doc='glGetPerfMonitorGroupStringAMD(GLuint(group), GLsizei(bufSize), GLsizeiArray(length), GLcharArray(groupString)) -> None',
argNames=('group','bufSize','length','groupString',),
deprecated=_DEPRECATED,
)

glGetPerfMonitorCounterStringAMD = platform.createExtensionFunction( 
'glGetPerfMonitorCounterStringAMD',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLuint,constants.GLsizei,arrays.GLsizeiArray,arrays.GLcharArray,),
doc='glGetPerfMonitorCounterStringAMD(GLuint(group), GLuint(counter), GLsizei(bufSize), GLsizeiArray(length), GLcharArray(counterString)) -> None',
argNames=('group','counter','bufSize','length','counterString',),
deprecated=_DEPRECATED,
)

glGetPerfMonitorCounterInfoAMD = platform.createExtensionFunction( 
'glGetPerfMonitorCounterInfoAMD',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLuint,constants.GLenum,ctypes.POINTER(constants.void),),
doc='glGetPerfMonitorCounterInfoAMD(GLuint(group), GLuint(counter), GLenum(pname), POINTER(constants.void)(data)) -> None',
argNames=('group','counter','pname','data',),
deprecated=_DEPRECATED,
)

glGenPerfMonitorsAMD = platform.createExtensionFunction( 
'glGenPerfMonitorsAMD',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLsizei,arrays.GLuintArray,),
doc='glGenPerfMonitorsAMD(GLsizei(n), GLuintArray(monitors)) -> None',
argNames=('n','monitors',),
deprecated=_DEPRECATED,
)

glDeletePerfMonitorsAMD = platform.createExtensionFunction( 
'glDeletePerfMonitorsAMD',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLsizei,arrays.GLuintArray,),
doc='glDeletePerfMonitorsAMD(GLsizei(n), GLuintArray(monitors)) -> None',
argNames=('n','monitors',),
deprecated=_DEPRECATED,
)

glSelectPerfMonitorCountersAMD = platform.createExtensionFunction( 
'glSelectPerfMonitorCountersAMD',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLboolean,constants.GLuint,constants.GLint,arrays.GLuintArray,),
doc='glSelectPerfMonitorCountersAMD(GLuint(monitor), GLboolean(enable), GLuint(group), GLint(numCounters), GLuintArray(counterList)) -> None',
argNames=('monitor','enable','group','numCounters','counterList',),
deprecated=_DEPRECATED,
)

glBeginPerfMonitorAMD = platform.createExtensionFunction( 
'glBeginPerfMonitorAMD',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,),
doc='glBeginPerfMonitorAMD(GLuint(monitor)) -> None',
argNames=('monitor',),
deprecated=_DEPRECATED,
)

glEndPerfMonitorAMD = platform.createExtensionFunction( 
'glEndPerfMonitorAMD',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,),
doc='glEndPerfMonitorAMD(GLuint(monitor)) -> None',
argNames=('monitor',),
deprecated=_DEPRECATED,
)

glGetPerfMonitorCounterDataAMD = platform.createExtensionFunction( 
'glGetPerfMonitorCounterDataAMD',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLenum,constants.GLsizei,arrays.GLuintArray,arrays.GLintArray,),
doc='glGetPerfMonitorCounterDataAMD(GLuint(monitor), GLenum(pname), GLsizei(dataSize), GLuintArray(data), GLintArray(bytesWritten)) -> None',
argNames=('monitor','pname','dataSize','data','bytesWritten',),
deprecated=_DEPRECATED,
)


def glInitPerformanceMonitorAMD():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension SGIX.async_pixel

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_SGIX_async_pixel'
_DEPRECATED = False
GL_ASYNC_TEX_IMAGE_SGIX = constant.Constant( 'GL_ASYNC_TEX_IMAGE_SGIX', 0x835C )
glget.addGLGetConstant( GL_ASYNC_TEX_IMAGE_SGIX, (1,) )
GL_ASYNC_DRAW_PIXELS_SGIX = constant.Constant( 'GL_ASYNC_DRAW_PIXELS_SGIX', 0x835D )
glget.addGLGetConstant( GL_ASYNC_DRAW_PIXELS_SGIX, (1,) )
GL_ASYNC_READ_PIXELS_SGIX = constant.Constant( 'GL_ASYNC_READ_PIXELS_SGIX', 0x835E )
glget.addGLGetConstant( GL_ASYNC_READ_PIXELS_SGIX, (1,) )
GL_MAX_ASYNC_TEX_IMAGE_SGIX = constant.Constant( 'GL_MAX_ASYNC_TEX_IMAGE_SGIX', 0x835F )
glget.addGLGetConstant( GL_MAX_ASYNC_TEX_IMAGE_SGIX, (1,) )
GL_MAX_ASYNC_DRAW_PIXELS_SGIX = constant.Constant( 'GL_MAX_ASYNC_DRAW_PIXELS_SGIX', 0x8360 )
glget.addGLGetConstant( GL_MAX_ASYNC_DRAW_PIXELS_SGIX, (1,) )
GL_MAX_ASYNC_READ_PIXELS_SGIX = constant.Constant( 'GL_MAX_ASYNC_READ_PIXELS_SGIX', 0x8361 )
glget.addGLGetConstant( GL_MAX_ASYNC_READ_PIXELS_SGIX, (1,) )


def glInitAsyncPixelSGIX():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension EXT.texture_cube_map

This module customises the behaviour of the 
OpenGL.raw.GL.EXT.texture_cube_map to provide a more 
Python-friendly API

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/texture_cube_map.txt
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions, wrapper
from OpenGL.GL import glget
import ctypes
from OpenGL.raw.GL.EXT.texture_cube_map import *
### END AUTOGENERATED SECTION


'''OpenGL extension ARB.copy_buffer

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_ARB_copy_buffer'
_DEPRECATED = False
GL_COPY_READ_BUFFER = constant.Constant( 'GL_COPY_READ_BUFFER', 0x8F36 )
GL_COPY_WRITE_BUFFER = constant.Constant( 'GL_COPY_WRITE_BUFFER', 0x8F37 )
glCopyBufferSubData = platform.createExtensionFunction( 
'glCopyBufferSubData',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,constants.GLintptr,constants.GLintptr,constants.GLsizeiptr,),
doc='glCopyBufferSubData(GLenum(readTarget), GLenum(writeTarget), GLintptr(readOffset), GLintptr(writeOffset), GLsizeiptr(size)) -> None',
argNames=('readTarget','writeTarget','readOffset','writeOffset','size',),
deprecated=_DEPRECATED,
)


def glInitCopyBufferARB():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension ARB.map_buffer_range

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_ARB_map_buffer_range'
_DEPRECATED = False
GL_MAP_READ_BIT = constant.Constant( 'GL_MAP_READ_BIT', 0x1 )
GL_MAP_WRITE_BIT = constant.Constant( 'GL_MAP_WRITE_BIT', 0x2 )
GL_MAP_INVALIDATE_RANGE_BIT = constant.Constant( 'GL_MAP_INVALIDATE_RANGE_BIT', 0x4 )
GL_MAP_INVALIDATE_BUFFER_BIT = constant.Constant( 'GL_MAP_INVALIDATE_BUFFER_BIT', 0x8 )
GL_MAP_FLUSH_EXPLICIT_BIT = constant.Constant( 'GL_MAP_FLUSH_EXPLICIT_BIT', 0x10 )
GL_MAP_UNSYNCHRONIZED_BIT = constant.Constant( 'GL_MAP_UNSYNCHRONIZED_BIT', 0x20 )
glMapBufferRange = platform.createExtensionFunction( 
'glMapBufferRange',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=ctypes.c_void_p, 
argTypes=(constants.GLenum,constants.GLintptr,constants.GLsizeiptr,constants.GLbitfield,),
doc='glMapBufferRange(GLenum(target), GLintptr(offset), GLsizeiptr(length), GLbitfield(access)) -> ctypes.c_void_p',
argNames=('target','offset','length','access',),
deprecated=_DEPRECATED,
)

glFlushMappedBufferRange = platform.createExtensionFunction( 
'glFlushMappedBufferRange',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLintptr,constants.GLsizeiptr,),
doc='glFlushMappedBufferRange(GLenum(target), GLintptr(offset), GLsizeiptr(length)) -> None',
argNames=('target','offset','length',),
deprecated=_DEPRECATED,
)


def glInitMapBufferRangeARB():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension APPLE.texture_range

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_APPLE_texture_range'
_DEPRECATED = False
GL_TEXTURE_RANGE_LENGTH_APPLE = constant.Constant( 'GL_TEXTURE_RANGE_LENGTH_APPLE', 0x85B7 )
GL_TEXTURE_RANGE_POINTER_APPLE = constant.Constant( 'GL_TEXTURE_RANGE_POINTER_APPLE', 0x85B8 )
GL_TEXTURE_STORAGE_HINT_APPLE = constant.Constant( 'GL_TEXTURE_STORAGE_HINT_APPLE', 0x85BC )
GL_STORAGE_PRIVATE_APPLE = constant.Constant( 'GL_STORAGE_PRIVATE_APPLE', 0x85BD )
glTextureRangeAPPLE = platform.createExtensionFunction( 
'glTextureRangeAPPLE',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLsizei,ctypes.c_void_p,),
doc='glTextureRangeAPPLE(GLenum(target), GLsizei(length), c_void_p(pointer)) -> None',
argNames=('target','length','pointer',),
deprecated=_DEPRECATED,
)

glGetTexParameterPointervAPPLE = platform.createExtensionFunction( 
'glGetTexParameterPointervAPPLE',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,ctypes.POINTER(ctypes.c_void_p),),
doc='glGetTexParameterPointervAPPLE(GLenum(target), GLenum(pname), POINTER(ctypes.c_void_p)(params)) -> None',
argNames=('target','pname','params',),
deprecated=_DEPRECATED,
)


def glInitTextureRangeAPPLE():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension EXT.packed_pixels

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_packed_pixels'
_DEPRECATED = False
GL_UNSIGNED_BYTE_3_3_2_EXT = constant.Constant( 'GL_UNSIGNED_BYTE_3_3_2_EXT', 0x8032 )
GL_UNSIGNED_SHORT_4_4_4_4_EXT = constant.Constant( 'GL_UNSIGNED_SHORT_4_4_4_4_EXT', 0x8033 )
GL_UNSIGNED_SHORT_5_5_5_1_EXT = constant.Constant( 'GL_UNSIGNED_SHORT_5_5_5_1_EXT', 0x8034 )
GL_UNSIGNED_INT_8_8_8_8_EXT = constant.Constant( 'GL_UNSIGNED_INT_8_8_8_8_EXT', 0x8035 )
GL_UNSIGNED_INT_10_10_10_2_EXT = constant.Constant( 'GL_UNSIGNED_INT_10_10_10_2_EXT', 0x8036 )


def glInitPackedPixelsEXT():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



"""OpenGL-wide constant types (not OpenGL.GL-specific)

These are basically the fundamental data-types that OpenGL 
uses (note, doesn't include the OpenGL-ES types!)
"""
import ctypes
from OpenGL.constant import Constant

GL_FALSE = Constant( 'GL_FALSE', 0x0 )
GL_TRUE = Constant( 'GL_TRUE', 0x1 )
GL_BYTE = Constant( 'GL_BYTE', 0x1400 )
GL_UNSIGNED_BYTE = Constant( 'GL_UNSIGNED_BYTE', 0x1401 )
GL_SHORT = Constant( 'GL_SHORT', 0x1402 )
GL_UNSIGNED_SHORT = Constant( 'GL_UNSIGNED_SHORT', 0x1403 )
GL_INT = Constant( 'GL_INT', 0x1404 )
GL_UNSIGNED_INT = Constant( 'GL_UNSIGNED_INT', 0x1405 )
GL_UNSIGNED_INT64 = Constant( 'GL_UNSIGNED_INT64_AMD', 0x8BC2 )
GL_FLOAT = Constant( 'GL_FLOAT', 0x1406 )
GL_DOUBLE = Constant( 'GL_DOUBLE', 0x140a )
GL_CHAR = str
GL_HALF_NV = Constant( 'GL_HALF_NV', 0x1401 )

ctypes_version = [int(i) for i in ctypes.__version__.split('.')[:3]]

# Basic OpenGL data-types as ctypes declarations...
def _defineType( name, baseType, convertFunc = long ):
    import OpenGL
    do_wrapping = (
        OpenGL.ALLOW_NUMPY_SCALARS or # explicitly require
        (( # or we are using Python 2.5.x ctypes which doesn't support uint type numpy scalars
            ctypes_version < [1,1,0]
            and baseType in (ctypes.c_uint,ctypes.c_uint64,ctypes.c_ulong,ctypes.c_ushort)
        ) or
        ( # or we are using Python 2.5.x (x < 2) ctypes which doesn't support any numpy int scalars
            ctypes_version < [1,0,2]
            and baseType in (ctypes.c_int,ctypes.c_int64,ctypes.c_long,ctypes.c_short)
        ))
    )
    if do_wrapping:
        original = baseType.from_param
        if not getattr( original, 'from_param_numpy_scalar', False ):
            def from_param( x, typeCode=None ):
                try:
                    return original( x )
                except TypeError, err:
                    try:
                        return original( convertFunc(x) )
                    except TypeError, err2:
                        raise err
            from_param = staticmethod( from_param )
            setattr( baseType, 'from_param', from_param )
            baseType.from_param_numpy_scalar = True
        return baseType
    else:
        return baseType

GLvoid = None
GLboolean = _defineType( 'GLboolean', ctypes.c_ubyte, bool )
GLenum = _defineType( 'GLenum', ctypes.c_uint )

GLfloat = _defineType( 'GLfloat', ctypes.c_float, float )
GLfloat_2 = GLfloat * 2
GLfloat_3 = GLfloat * 3
GLfloat_4 = GLfloat * 4
GLdouble = _defineType( 'GLdouble', ctypes.c_double, float )
GLdouble_2 = GLdouble * 2
GLdouble_3 = GLdouble * 3
GLdouble_4 = GLdouble * 4

GLbyte = ctypes.c_byte
GLshort = _defineType( 'GLshort', ctypes.c_short, int )
GLint = _defineType( 'GLint', ctypes.c_int, int )
GLuint = _defineType( 'GLuint', ctypes.c_uint, long )

GLsizei = _defineType( 'GLsizei', ctypes.c_int, int )

GLubyte = ctypes.c_ubyte
GLubyte_3 = GLubyte * 3
GLushort = _defineType( 'GLushort', ctypes.c_ushort, int )
GLhandleARB = _defineType( 'GLhandleARB', ctypes.c_uint, long )
GLhandle = _defineType( 'GLhandle', ctypes.c_uint, long )

GLchar = GLcharARB = ctypes.c_char

GLbitfield = _defineType( 'GLbitfield', ctypes.c_uint, long )

GLclampd = _defineType( 'GLclampd', ctypes.c_double, float )
GLclampf = _defineType( 'GLclampf', ctypes.c_float, float )

GLuint64 = GLuint64EXT = _defineType('GLuint64', ctypes.c_uint64, long )
GLint64 = GLint64EXT = _defineType('GLint64', ctypes.c_int64, long )

# ptrdiff_t, actually...
GLsizeiptrARB = GLsizeiptr = GLsizei
GLintptrARB = GLintptr = GLint
size_t = ctypes.c_ulong

void = None

GLhalfNV = GLhalfARB = ctypes.c_ushort

# GL.ARB.sync extension, GLsync is an opaque pointer to a struct 
# in the extensions header, basically just a "token" that can be 
# passed to the various operations...
GLsync = ctypes.c_void_p 


ARRAY_TYPE_TO_CONSTANT = [
    ('GLclampd', GL_DOUBLE),
    ('GLclampf', GL_FLOAT),
    ('GLfloat', GL_FLOAT),
    ('GLdouble', GL_DOUBLE),
    ('GLbyte', GL_BYTE),
    ('GLshort', GL_SHORT),
    ('GLint', GL_INT),
    ('GLubyte', GL_UNSIGNED_BYTE),
    ('GLushort', GL_UNSIGNED_SHORT),
    ('GLuint', GL_UNSIGNED_INT),
    ('GLenum', GL_UNSIGNED_INT),
]



'''OpenGL extension EXT.shared_texture_palette

This module customises the behaviour of the 
OpenGL.raw.GL.EXT.shared_texture_palette to provide a more 
Python-friendly API

Overview (from the spec)
	
	EXT_shared_texture_palette defines a shared texture palette which may be
	used in place of the texture object palettes provided by
	EXT_paletted_texture. This is useful for rapidly changing a palette
	common to many textures, rather than having to reload the new palette
	for each texture. The extension acts as a switch, causing all lookups
	that would normally be done on the texture's palette to instead use the
	shared palette.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/shared_texture_palette.txt
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions, wrapper
from OpenGL.GL import glget
import ctypes
from OpenGL.raw.GL.EXT.shared_texture_palette import *
### END AUTOGENERATED SECTION


'''OpenGL extension APPLE.object_purgeable

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_APPLE_object_purgeable'
_DEPRECATED = False
GL_BUFFER_OBJECT_APPLE = constant.Constant( 'GL_BUFFER_OBJECT_APPLE', 0x85B3 )
GL_RELEASED_APPLE = constant.Constant( 'GL_RELEASED_APPLE', 0x8A19 )
GL_VOLATILE_APPLE = constant.Constant( 'GL_VOLATILE_APPLE', 0x8A1A )
GL_RETAINED_APPLE = constant.Constant( 'GL_RETAINED_APPLE', 0x8A1B )
GL_UNDEFINED_APPLE = constant.Constant( 'GL_UNDEFINED_APPLE', 0x8A1C )
GL_PURGEABLE_APPLE = constant.Constant( 'GL_PURGEABLE_APPLE', 0x8A1D )
glObjectPurgeableAPPLE = platform.createExtensionFunction( 
'glObjectPurgeableAPPLE',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=constants.GLenum, 
argTypes=(constants.GLenum,constants.GLuint,constants.GLenum,),
doc='glObjectPurgeableAPPLE(GLenum(objectType), GLuint(name), GLenum(option)) -> constants.GLenum',
argNames=('objectType','name','option',),
deprecated=_DEPRECATED,
)

glObjectUnpurgeableAPPLE = platform.createExtensionFunction( 
'glObjectUnpurgeableAPPLE',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=constants.GLenum, 
argTypes=(constants.GLenum,constants.GLuint,constants.GLenum,),
doc='glObjectUnpurgeableAPPLE(GLenum(objectType), GLuint(name), GLenum(option)) -> constants.GLenum',
argNames=('objectType','name','option',),
deprecated=_DEPRECATED,
)

glGetObjectParameterivAPPLE = platform.createExtensionFunction( 
'glGetObjectParameterivAPPLE',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLuint,constants.GLenum,arrays.GLintArray,),
doc='glGetObjectParameterivAPPLE(GLenum(objectType), GLuint(name), GLenum(pname), GLintArray(params)) -> None',
argNames=('objectType','name','pname','params',),
deprecated=_DEPRECATED,
)


def glInitObjectPurgeableAPPLE():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension IBM.multimode_draw_arrays

This module customises the behaviour of the 
OpenGL.raw.GL.IBM.multimode_draw_arrays to provide a more 
Python-friendly API

Overview (from the spec)
	
	These functions behave identically to the standard OpenGL 1.1 functions
	glDrawArrays() and glDrawElements() except they handle multiple lists of
	vertices and multiple primitive modes in one call. Their main purpose is
	to allow one function call to render more than one primitive regardless
	of the primitive mode. 
	
	This extension is similar to the EXT_multi_draw_arrays extension 
	except that it accomodates the specification of a  unique mode for
	each primitive.
	

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/IBM/multimode_draw_arrays.txt
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions, wrapper
from OpenGL.GL import glget
import ctypes
from OpenGL.raw.GL.IBM.multimode_draw_arrays import *
### END AUTOGENERATED SECTION


'''OpenGL extension ARB.window_pos

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_ARB_window_pos'
_DEPRECATED = False

glWindowPos2dARB = platform.createExtensionFunction( 
'glWindowPos2dARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLdouble,constants.GLdouble,),
doc='glWindowPos2dARB(GLdouble(x), GLdouble(y)) -> None',
argNames=('x','y',),
deprecated=_DEPRECATED,
)

glWindowPos2dvARB = platform.createExtensionFunction( 
'glWindowPos2dvARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(arrays.GLdoubleArray,),
doc='glWindowPos2dvARB(GLdoubleArray(v)) -> None',
argNames=('v',),
deprecated=_DEPRECATED,
)

glWindowPos2fARB = platform.createExtensionFunction( 
'glWindowPos2fARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLfloat,constants.GLfloat,),
doc='glWindowPos2fARB(GLfloat(x), GLfloat(y)) -> None',
argNames=('x','y',),
deprecated=_DEPRECATED,
)

glWindowPos2fvARB = platform.createExtensionFunction( 
'glWindowPos2fvARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(arrays.GLfloatArray,),
doc='glWindowPos2fvARB(GLfloatArray(v)) -> None',
argNames=('v',),
deprecated=_DEPRECATED,
)

glWindowPos2iARB = platform.createExtensionFunction( 
'glWindowPos2iARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLint,constants.GLint,),
doc='glWindowPos2iARB(GLint(x), GLint(y)) -> None',
argNames=('x','y',),
deprecated=_DEPRECATED,
)

glWindowPos2ivARB = platform.createExtensionFunction( 
'glWindowPos2ivARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(arrays.GLintArray,),
doc='glWindowPos2ivARB(GLintArray(v)) -> None',
argNames=('v',),
deprecated=_DEPRECATED,
)

glWindowPos2sARB = platform.createExtensionFunction( 
'glWindowPos2sARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLshort,constants.GLshort,),
doc='glWindowPos2sARB(GLshort(x), GLshort(y)) -> None',
argNames=('x','y',),
deprecated=_DEPRECATED,
)

glWindowPos2svARB = platform.createExtensionFunction( 
'glWindowPos2svARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(arrays.GLshortArray,),
doc='glWindowPos2svARB(GLshortArray(v)) -> None',
argNames=('v',),
deprecated=_DEPRECATED,
)

glWindowPos3dARB = platform.createExtensionFunction( 
'glWindowPos3dARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLdouble,constants.GLdouble,constants.GLdouble,),
doc='glWindowPos3dARB(GLdouble(x), GLdouble(y), GLdouble(z)) -> None',
argNames=('x','y','z',),
deprecated=_DEPRECATED,
)

glWindowPos3dvARB = platform.createExtensionFunction( 
'glWindowPos3dvARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(arrays.GLdoubleArray,),
doc='glWindowPos3dvARB(GLdoubleArray(v)) -> None',
argNames=('v',),
deprecated=_DEPRECATED,
)

glWindowPos3fARB = platform.createExtensionFunction( 
'glWindowPos3fARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLfloat,constants.GLfloat,constants.GLfloat,),
doc='glWindowPos3fARB(GLfloat(x), GLfloat(y), GLfloat(z)) -> None',
argNames=('x','y','z',),
deprecated=_DEPRECATED,
)

glWindowPos3fvARB = platform.createExtensionFunction( 
'glWindowPos3fvARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(arrays.GLfloatArray,),
doc='glWindowPos3fvARB(GLfloatArray(v)) -> None',
argNames=('v',),
deprecated=_DEPRECATED,
)

glWindowPos3iARB = platform.createExtensionFunction( 
'glWindowPos3iARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLint,constants.GLint,constants.GLint,),
doc='glWindowPos3iARB(GLint(x), GLint(y), GLint(z)) -> None',
argNames=('x','y','z',),
deprecated=_DEPRECATED,
)

glWindowPos3ivARB = platform.createExtensionFunction( 
'glWindowPos3ivARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(arrays.GLintArray,),
doc='glWindowPos3ivARB(GLintArray(v)) -> None',
argNames=('v',),
deprecated=_DEPRECATED,
)

glWindowPos3sARB = platform.createExtensionFunction( 
'glWindowPos3sARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLshort,constants.GLshort,constants.GLshort,),
doc='glWindowPos3sARB(GLshort(x), GLshort(y), GLshort(z)) -> None',
argNames=('x','y','z',),
deprecated=_DEPRECATED,
)

glWindowPos3svARB = platform.createExtensionFunction( 
'glWindowPos3svARB',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(arrays.GLshortArray,),
doc='glWindowPos3svARB(GLshortArray(v)) -> None',
argNames=('v',),
deprecated=_DEPRECATED,
)


def glInitWindowPosARB():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension APPLE.flush_buffer_range

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_APPLE_flush_buffer_range'
_DEPRECATED = False
GL_BUFFER_SERIALIZED_MODIFY_APPLE = constant.Constant( 'GL_BUFFER_SERIALIZED_MODIFY_APPLE', 0x8A12 )
GL_BUFFER_FLUSHING_UNMAP_APPLE = constant.Constant( 'GL_BUFFER_FLUSHING_UNMAP_APPLE', 0x8A13 )
glBufferParameteriAPPLE = platform.createExtensionFunction( 
'glBufferParameteriAPPLE',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLenum,constants.GLint,),
doc='glBufferParameteriAPPLE(GLenum(target), GLenum(pname), GLint(param)) -> None',
argNames=('target','pname','param',),
deprecated=_DEPRECATED,
)

glFlushMappedBufferRangeAPPLE = platform.createExtensionFunction( 
'glFlushMappedBufferRangeAPPLE',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLenum,constants.GLintptr,constants.GLsizeiptr,),
doc='glFlushMappedBufferRangeAPPLE(GLenum(target), GLintptr(offset), GLsizeiptr(size)) -> None',
argNames=('target','offset','size',),
deprecated=_DEPRECATED,
)


def glInitFlushBufferRangeAPPLE():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension DFX.tbuffer

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_DFX_tbuffer'
_DEPRECATED = False

glTbufferMask3DFX = platform.createExtensionFunction( 
'glTbufferMask3DFX',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,),
doc='glTbufferMask3DFX(GLuint(mask)) -> None',
argNames=('mask',),
deprecated=_DEPRECATED,
)


def glInitTbufferDFX():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )



"""OpenGL.GL, the core GL library and extensions to it"""
from OpenGL.raw.GL import *
from OpenGL.raw.GL.annotations import *

from OpenGL.GL.pointers import *
from OpenGL.GL.glget import *
from OpenGL.GL.images import *
from OpenGL.GL.exceptional import *
from OpenGL.error import *

from OpenGL.GL.VERSION.GL_1_2 import *
from OpenGL.GL.VERSION.GL_1_3 import *
from OpenGL.GL.VERSION.GL_1_4 import *
from OpenGL.GL.VERSION.GL_1_5 import *
from OpenGL.GL.VERSION.GL_2_0 import *
from OpenGL.GL.VERSION.GL_2_1 import *
from OpenGL.GL.VERSION.GL_3_0 import *
from OpenGL.GL.VERSION.GL_3_1 import *
from OpenGL.GL.VERSION.GL_3_2 import *

GLerror = GLError

# Now the aliases...
glRotate = glRotated
glTranslate = glTranslated
glLight = glLightfv
glTexCoord = glTexCoord2d
glScale = glScaled
#glColor = glColor3f
glNormal = glNormal3d



'''OpenGL extension APPLE.vertex_program_evaluators

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_APPLE_vertex_program_evaluators'
_DEPRECATED = False
GL_VERTEX_ATTRIB_MAP1_APPLE = constant.Constant( 'GL_VERTEX_ATTRIB_MAP1_APPLE', 0x8A00 )
GL_VERTEX_ATTRIB_MAP2_APPLE = constant.Constant( 'GL_VERTEX_ATTRIB_MAP2_APPLE', 0x8A01 )
GL_VERTEX_ATTRIB_MAP1_SIZE_APPLE = constant.Constant( 'GL_VERTEX_ATTRIB_MAP1_SIZE_APPLE', 0x8A02 )
GL_VERTEX_ATTRIB_MAP1_COEFF_APPLE = constant.Constant( 'GL_VERTEX_ATTRIB_MAP1_COEFF_APPLE', 0x8A03 )
GL_VERTEX_ATTRIB_MAP1_ORDER_APPLE = constant.Constant( 'GL_VERTEX_ATTRIB_MAP1_ORDER_APPLE', 0x8A04 )
GL_VERTEX_ATTRIB_MAP1_DOMAIN_APPLE = constant.Constant( 'GL_VERTEX_ATTRIB_MAP1_DOMAIN_APPLE', 0x8A05 )
GL_VERTEX_ATTRIB_MAP2_SIZE_APPLE = constant.Constant( 'GL_VERTEX_ATTRIB_MAP2_SIZE_APPLE', 0x8A06 )
GL_VERTEX_ATTRIB_MAP2_COEFF_APPLE = constant.Constant( 'GL_VERTEX_ATTRIB_MAP2_COEFF_APPLE', 0x8A07 )
GL_VERTEX_ATTRIB_MAP2_ORDER_APPLE = constant.Constant( 'GL_VERTEX_ATTRIB_MAP2_ORDER_APPLE', 0x8A08 )
GL_VERTEX_ATTRIB_MAP2_DOMAIN_APPLE = constant.Constant( 'GL_VERTEX_ATTRIB_MAP2_DOMAIN_APPLE', 0x8A09 )
glEnableVertexAttribAPPLE = platform.createExtensionFunction( 
'glEnableVertexAttribAPPLE',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLenum,),
doc='glEnableVertexAttribAPPLE(GLuint(index), GLenum(pname)) -> None',
argNames=('index','pname',),
deprecated=_DEPRECATED,
)

glDisableVertexAttribAPPLE = platform.createExtensionFunction( 
'glDisableVertexAttribAPPLE',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLenum,),
doc='glDisableVertexAttribAPPLE(GLuint(index), GLenum(pname)) -> None',
argNames=('index','pname',),
deprecated=_DEPRECATED,
)

glIsVertexAttribEnabledAPPLE = platform.createExtensionFunction( 
'glIsVertexAttribEnabledAPPLE',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=constants.GLboolean, 
argTypes=(constants.GLuint,constants.GLenum,),
doc='glIsVertexAttribEnabledAPPLE(GLuint(index), GLenum(pname)) -> constants.GLboolean',
argNames=('index','pname',),
deprecated=_DEPRECATED,
)

glMapVertexAttrib1dAPPLE = platform.createExtensionFunction( 
'glMapVertexAttrib1dAPPLE',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLuint,constants.GLdouble,constants.GLdouble,constants.GLint,constants.GLint,arrays.GLdoubleArray,),
doc='glMapVertexAttrib1dAPPLE(GLuint(index), GLuint(size), GLdouble(u1), GLdouble(u2), GLint(stride), GLint(order), GLdoubleArray(points)) -> None',
argNames=('index','size','u1','u2','stride','order','points',),
deprecated=_DEPRECATED,
)

glMapVertexAttrib1fAPPLE = platform.createExtensionFunction( 
'glMapVertexAttrib1fAPPLE',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLuint,constants.GLfloat,constants.GLfloat,constants.GLint,constants.GLint,arrays.GLfloatArray,),
doc='glMapVertexAttrib1fAPPLE(GLuint(index), GLuint(size), GLfloat(u1), GLfloat(u2), GLint(stride), GLint(order), GLfloatArray(points)) -> None',
argNames=('index','size','u1','u2','stride','order','points',),
deprecated=_DEPRECATED,
)

glMapVertexAttrib2dAPPLE = platform.createExtensionFunction( 
'glMapVertexAttrib2dAPPLE',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLuint,constants.GLdouble,constants.GLdouble,constants.GLint,constants.GLint,constants.GLdouble,constants.GLdouble,constants.GLint,constants.GLint,arrays.GLdoubleArray,),
doc='glMapVertexAttrib2dAPPLE(GLuint(index), GLuint(size), GLdouble(u1), GLdouble(u2), GLint(ustride), GLint(uorder), GLdouble(v1), GLdouble(v2), GLint(vstride), GLint(vorder), GLdoubleArray(points)) -> None',
argNames=('index','size','u1','u2','ustride','uorder','v1','v2','vstride','vorder','points',),
deprecated=_DEPRECATED,
)

glMapVertexAttrib2fAPPLE = platform.createExtensionFunction( 
'glMapVertexAttrib2fAPPLE',dll=platform.GL,
extension=EXTENSION_NAME,
resultType=None, 
argTypes=(constants.GLuint,constants.GLuint,constants.GLfloat,constants.GLfloat,constants.GLint,constants.GLint,constants.GLfloat,constants.GLfloat,constants.GLint,constants.GLint,arrays.GLfloatArray,),
doc='glMapVertexAttrib2fAPPLE(GLuint(index), GLuint(size), GLfloat(u1), GLfloat(u2), GLint(ustride), GLint(uorder), GLfloat(v1), GLfloat(v2), GLint(vstride), GLint(vorder), GLfloatArray(points)) -> None',
argNames=('index','size','u1','u2','ustride','uorder','v1','v2','vstride','vorder','points',),
deprecated=_DEPRECATED,
)


def glInitVertexProgramEvaluatorsAPPLE():
    '''Return boolean indicating whether this extension is available'''
    return extensions.hasGLExtension( EXTENSION_NAME )
